@@ -63,6 +63,14 @@ namespace details {
 # endif
     template <typename Ty>
     using  is_float_or_half = _is_float_or_half<typename _CG_STL_NAMESPACE::remove_cv<Ty>::type>;
+
+    // Non-STL utility templates 
+    template <typename Ty>
+    using remove_qual = typename _CG_STL_NAMESPACE::remove_cv<typename _CG_STL_NAMESPACE::remove_reference<Ty>::type>::type;
+
+    template <typename TyLhs, typename TyRhs>
+    using is_op_type_same = _CG_STL_NAMESPACE::is_same<remove_qual<TyLhs>, remove_qual<TyRhs>
+    >;
 #endif
 
     template <typename TyTrunc>
@@ -72,11 +80,6 @@ namespace details {
                 (TyTrunc)index.x;
     }
 
-#ifdef _CG_CPP11_FEATURES
-    template <typename Ty>
-    using remove_qual = typename _CG_STL_NAMESPACE::remove_cv<typename _CG_STL_NAMESPACE::remove_reference<Ty>::type>::type;
-# endif
-
     class _coalesced_group_data_access {
     public:
         // Retrieve mask of coalesced groups
@@ -322,86 +325,6 @@ namespace details {
 #endif //_CG_CPP11_FEATURES
     };
 
-#ifdef _CG_CPP11_FEATURES
-    template <unsigned int numWarps>
-    struct copy_channel {
-        char* channel_ptr;
-        unsigned int* sync_location;
-        size_t channel_size;
-
-        // One thread sending to any all other threads, it has to wait for all warps (including its own).
-        struct send_one_to_many {
-            _CG_STATIC_CONST_DECL unsigned int wait_count = numWarps;
-            _CG_STATIC_QUALIFIER void post_iter_release(unsigned int thread_idx, unsigned int* sync_location, unsigned int sync_mask) {
-                __syncwarp(sync_mask);
-                details::sync_warps_release(sync_location, thread_idx == 0);
-            }
-        };
-
-        // One warp sending to all other warps, it has to wait for all other warps.
-        struct send_many_to_many {
-            _CG_STATIC_CONST_DECL unsigned int wait_count = numWarps - 1;
-            _CG_STATIC_QUALIFIER void post_iter_release(unsigned int thread_idx, unsigned int* sync_location, unsigned int sync_mask) {
-                __syncwarp(sync_mask);
-                details::sync_warps_release(sync_location, thread_idx == 0);
-            }
-        };
-
-        // One warp is receiving and all other warps are sending to that warp, they have to wait for that one warp.
-        struct send_many_to_one {
-            _CG_STATIC_CONST_DECL unsigned int wait_count = 1;
-            _CG_STATIC_QUALIFIER void post_iter_release(unsigned int thread_idx, unsigned int* sync_location, unsigned int sync_mask) {
-                // Wait for all warps to finish and let the last warp release all threads.
-                if (details::sync_warps_last_releases(numWarps, sync_location, thread_idx)) {
-                    details::sync_warps_release(sync_location, thread_idx == 0);
-                }
-            }
-        };
-
-        template <unsigned int ThreadCnt, size_t ValSize, typename SendDetails>
-        _CG_QUALIFIER void _send_value_internal(char* val_ptr, unsigned int thread_idx, bool active, unsigned int sync_mask) {
-            size_t thread_offset = thread_idx * sizeof(int);
-
-            for (size_t i = 0; i < ValSize; i += channel_size) {
-                size_t bytes_left = ValSize - i;
-                size_t copy_chunk = min(bytes_left, channel_size);
-
-                details::sync_warps_wait_for_count(SendDetails::wait_count, sync_location);
-                if (active) {
-                    #pragma unroll 1
-                    for (size_t j = thread_offset; j < copy_chunk ; j += sizeof(int) * ThreadCnt) {
-                        size_t my_bytes_left = copy_chunk - j;
-                        memcpy(channel_ptr + j, val_ptr + i + j, min(my_bytes_left, sizeof(int)));
-                    }
-                }
-                SendDetails::post_iter_release(thread_idx, sync_location, sync_mask);
-            }
-        }
-
-
-        template <typename TyVal, unsigned int ThreadCnt, typename SendDetails>
-        _CG_QUALIFIER void send_value(TyVal& val, unsigned int thread_idx, bool active = true, unsigned int sync_mask = 0xFFFFFFFF) {
-            _send_value_internal<ThreadCnt, sizeof(TyVal), SendDetails>(reinterpret_cast<char*>(&val), thread_idx, active, sync_mask);
-        }
-
-        template <size_t ValSize>
-        _CG_QUALIFIER void _receive_value_internal(char* val_ptr, bool warp_master, bool active, unsigned int sync_mask) {
-            for (size_t i = 0; i < ValSize; i += channel_size) {
-                size_t bytes_left = ValSize - i;
-                details::sync_warps_wait_for_release(sync_location, warp_master, sync_mask);
-                if (active) {
-                    memcpy(val_ptr + i, channel_ptr, min(bytes_left, channel_size));
-                }
-            }
-        }
-
-        template <typename TyVal>
-        _CG_QUALIFIER void receive_value(TyVal& val, bool warp_master, bool active = true, unsigned int sync_mask = 0xFFFFFFFF) {
-            _receive_value_internal<sizeof(TyVal)>(reinterpret_cast<char*>(&val), warp_master, active, sync_mask);
-        }
-    };
-#endif //_CG_CPP11_FEATURES
-
     namespace multi_grid {
         struct multi_grid_functions;
     };
@@ -581,6 +504,79 @@ namespace details {
 #endif
     }
 
+#if defined(_CG_CPP11_FEATURES) && defined(_CG_ABI_EXPERIMENTAL)
+    template <unsigned int numWarps>
+    struct copy_channel {
+        char* channel_ptr;
+        barrier_t* sync_location;
+        size_t channel_size;
+
+        // One warp sending to all other warps, it has to wait for all other warps.
+        struct send_many_to_many {
+            _CG_STATIC_CONST_DECL wait_for_warps_kind wait_kind = wait_for_all_other_warps;
+            _CG_STATIC_QUALIFIER void post_iter_release(unsigned int thread_idx, barrier_t* sync_location) {
+                __syncwarp(0xFFFFFFFF);
+                details::sync_warps_release(sync_location, thread_idx == 0, cta::thread_rank(), numWarps);
+            }
+        };
+
+        // One warp is receiving and all other warps are sending to that warp, they have to wait for that one warp.
+        struct send_many_to_one {
+            _CG_STATIC_CONST_DECL wait_for_warps_kind wait_kind = wait_for_specific_warp;
+            _CG_STATIC_QUALIFIER void post_iter_release(unsigned int thread_idx, barrier_t* sync_location) {
+                // Wait for all warps to finish and let the last warp release all threads.
+                if (details::sync_warps_last_releases(sync_location, cta::thread_rank(), numWarps)) {
+                    details::sync_warps_release(sync_location, thread_idx == 0, cta::thread_rank(), numWarps);
+                }
+            }
+        };
+
+        template <unsigned int ThreadCnt, size_t ValSize, typename SendDetails>
+        _CG_QUALIFIER void _send_value_internal(char* val_ptr, unsigned int thread_idx, unsigned int warp_id) {
+            size_t thread_offset = thread_idx * sizeof(int);
+
+            for (size_t i = 0; i < ValSize; i += channel_size) {
+                size_t bytes_left = ValSize - i;
+                size_t copy_chunk = min(bytes_left, channel_size);
+
+                details::sync_warps_wait_for_warps<SendDetails::wait_kind>(warp_id, sync_location, cta::thread_rank(), numWarps);
+                #pragma unroll 1
+                for (size_t j = thread_offset; j < copy_chunk ; j += sizeof(int) * ThreadCnt) {
+                    size_t my_bytes_left = copy_chunk - j;
+                    memcpy(channel_ptr + j, val_ptr + i + j, min(my_bytes_left, sizeof(int)));
+                }
+                SendDetails::post_iter_release(thread_idx, sync_location);
+            }
+        }
+
+
+        template <typename TyVal, unsigned int ThreadCnt, typename SendDetails>
+        _CG_QUALIFIER void send_value(TyVal& val, unsigned int thread_idx, unsigned int warp_id) {
+            _send_value_internal<ThreadCnt, sizeof(TyVal), SendDetails>(reinterpret_cast<char*>(&val), thread_idx, warp_id);
+        }
+
+        template <size_t ValSize>
+        _CG_QUALIFIER void _receive_value_internal(char* val_ptr, bool warp_master, bool active) {
+            for (size_t i = 0; i < ValSize; i += channel_size) {
+                size_t bytes_left = ValSize - i;
+                details::sync_warps_wait_for_release(sync_location, warp_master, cta::thread_rank(), numWarps);
+                if (active) {
+                    memcpy(val_ptr + i, channel_ptr, min(bytes_left, channel_size));
+                }
+            }
+        }
+
+        template <typename TyVal>
+        _CG_QUALIFIER void receive_value(TyVal& val, bool warp_master, bool active = true) {
+            _receive_value_internal<sizeof(TyVal)>(reinterpret_cast<char*>(&val), warp_master, active);
+        }
+    };
+
+    _CG_STATIC_QUALIFIER constexpr unsigned int log2(unsigned int x) {
+        return x == 1 ? 0 : 1 + log2(x / 2);
+    }
+#endif //_CG_CPP11_FEATURES
+
 }; // !Namespace internal
 
 _CG_END_NAMESPACE
