@@ -93,6 +93,7 @@
 #include <limits.h>
 #include <string.h>
 
+
 // __OCC_INLINE will be undefined at the end of this header
 //
 #ifdef __CUDACC__
@@ -573,9 +574,17 @@ typedef enum cudaOccPartitionedGCSupport_enum {
 /**
  * Max compute capability supported
  */
+
+
+
+
+
+
+
 #define __CUDA_OCC_MAJOR__ 8
 #define __CUDA_OCC_MINOR__ 6
 
+
 //////////////////////////////////////////
 //    Mathematical Helper Functions     //
 //////////////////////////////////////////
@@ -614,6 +623,9 @@ static __OCC_INLINE cudaOccError cudaOccSMemAllocationGranularity(int *limit, co
             value = 256;
             break;
         case 8:
+
+
+
             value = 128;
             break;
         default:
@@ -639,6 +651,9 @@ static __OCC_INLINE cudaOccError cudaOccRegAllocationMaxPerThread(int *limit, co
             value = 255;
         case 7:
         case 8:
+
+
+
             value = 256;
             break;
         default:
@@ -663,6 +678,9 @@ static __OCC_INLINE cudaOccError cudaOccRegAllocationGranularity(int *limit, con
         case 6:
         case 7:
         case 8:
+
+
+
             value = 256;
             break;
         default:
@@ -686,6 +704,9 @@ static __OCC_INLINE cudaOccError cudaOccSubPartitionsPerMultiprocessor(int *limi
         case 5:
         case 7:
         case 8:
+
+
+
             value = 4;
             break;
         case 6:
@@ -722,8 +743,24 @@ static __OCC_INLINE cudaOccError cudaOccMaxBlocksPerMultiprocessor(int* limit, c
             break;
         }
         case 8:
-            value = properties->computeMinor == 0 ? 32 : 16;
+            if (properties->computeMinor == 0) {
+                value = 32;
+            }
+
+
+
+
+
+
+            else {
+                value = 16;
+            }
             break;
+
+
+
+
+
         default:
             return CUDA_OCC_ERROR_UNKNOWN_DEVICE;
     }
@@ -838,6 +875,44 @@ static __OCC_INLINE cudaOccError cudaOccAlignUpShmemSizeVoltaPlus(size_t *shMemS
             }
         }
         break;
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
     default:
         return CUDA_OCC_ERROR_UNKNOWN_DEVICE;
     }
@@ -975,6 +1050,9 @@ static __OCC_INLINE cudaOccError cudaOccSMemPerBlock(size_t *limit, const cudaOc
             break;
         case 7:
         case 8:
+
+
+
             switch (shmemLimitConfig) {
                 default:
                 case FUNC_SHMEM_LIMIT_DEFAULT:
