@@ -62,6 +62,12 @@
 #include "cuda_runtime_api.h"
 #include "cuda_texture_types.h"
 
+#if defined(_WIN32)
+# define __DEPRECATED__ __declspec(deprecated)
+#else
+# define __DEPRECATED__  __attribute__((deprecated))
+#endif
+
 
 template <typename T>
 struct __nv_tex_rmet_ret { };
@@ -123,7 +129,7 @@ template<> struct __nv_tex_rmet_cast<ulong4> { typedef uint4 *type; };
 #endif /* !__LP64__ */
 
 template <typename T>
-static __forceinline__ __device__  typename __nv_tex_rmet_ret<T>::type tex1Dfetch(texture<T, cudaTextureType1D, cudaReadModeElementType> t, int x)
+static __DEPRECATED__ __forceinline__ __device__  typename __nv_tex_rmet_ret<T>::type tex1Dfetch(texture<T, cudaTextureType1D, cudaReadModeElementType> t, int x)
 {
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -154,7 +160,7 @@ template <> struct __nv_tex_rmnf_ret<short4> { typedef float4 type; };
 template <> struct __nv_tex_rmnf_ret<ushort4> { typedef float4 type; };
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1Dfetch(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, int x) 
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1Dfetch(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, int x) 
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -166,7 +172,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1Dfetch
 
 // tex1D
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1D(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1D(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -176,7 +182,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1D(text
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1D(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1D(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -189,7 +195,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1D(text
 
 //tex2D
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2D(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2D(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -200,7 +206,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2D(text
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2D(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2D(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -213,7 +219,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2D(text
 
 //tex1DLayered
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayered(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayered(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -223,7 +229,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayered(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayered(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -236,7 +242,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayer
 
 //tex2DLayered
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayered(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayered(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -246,7 +252,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayered(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayered(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -258,7 +264,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayer
 
 // tex3D
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3D(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3D(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -268,7 +274,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3D(text
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3D(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3D(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -280,7 +286,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3D(text
 
 // texCubemap
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -290,7 +296,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -392,7 +398,7 @@ static __device__ __forceinline__  typename __nv_tex2dgather_rmnf_ret<T>::type t
 
 // tex1DLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLod(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLod(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -402,7 +408,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLod(t
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLod(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLod(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -414,7 +420,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLod(t
 
 // tex2DLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLod(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLod(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -424,7 +430,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLod(t
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLod(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLod(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -436,7 +442,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLod(t
 
 // tex1DLayeredLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayeredLod(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayeredLod(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -446,7 +452,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayeredLod(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayeredLod(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -458,7 +464,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayer
 
 // tex2DLayeredLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayeredLod(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayeredLod(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -468,7 +474,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayeredLod(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayeredLod(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -480,7 +486,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayer
 
 // tex3DLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DLod(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DLod(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -490,7 +496,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DLod(t
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DLod(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DLod(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -502,7 +508,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DLod(t
 
 // texCubemapLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLod(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLod(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -512,7 +518,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLod(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLod(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -525,7 +531,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap
 
 // texCubemapLayered
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayered(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayered(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer)
 {
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -535,7 +541,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayered(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayered(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer)
 {
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -548,7 +554,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap
 
 // texCubemapLayeredLod
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayeredLod(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayeredLod(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -558,7 +564,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayeredLod(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer, float level)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayeredLod(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer, float level)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -571,7 +577,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap
 
 // texCubemapGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapGrad(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z, float4 dPdx, float4 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapGrad(texture<T, cudaTextureTypeCubemap, cudaReadModeElementType> t, float x, float y, float z, float4 dPdx, float4 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -581,7 +587,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapGrad(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z, float4 dPdx, float4 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapGrad(texture<T, cudaTextureTypeCubemap, cudaReadModeNormalizedFloat> t, float x, float y, float z, float4 dPdx, float4 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -594,7 +600,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap
 
 // texCubemapLayeredGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayeredGrad(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer, float4 dPdx, float4 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemapLayeredGrad(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeElementType> t, float x, float y, float z, int layer, float4 dPdx, float4 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -604,7 +610,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type texCubemap
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayeredGrad(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer, float4 dPdx, float4 dPdy) 
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemapLayeredGrad(texture<T, cudaTextureTypeCubemapLayered, cudaReadModeNormalizedFloat> t, float x, float y, float z, int layer, float4 dPdx, float4 dPdy) 
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -617,7 +623,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type texCubemap
 
 // tex1DGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DGrad(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x, float dPdx, float dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DGrad(texture<T, cudaTextureType1D, cudaReadModeElementType> t, float x, float dPdx, float dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -627,7 +633,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DGrad(
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DGrad(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x, float dPdx, float dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DGrad(texture<T, cudaTextureType1D, cudaReadModeNormalizedFloat> t, float x, float dPdx, float dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -640,7 +646,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DGrad(
 
 // tex2DGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DGrad(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y, float2 dPdx, float2 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DGrad(texture<T, cudaTextureType2D, cudaReadModeElementType> t, float x, float y, float2 dPdx, float2 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -650,7 +656,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DGrad(
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DGrad(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y, float2 dPdx, float2 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DGrad(texture<T, cudaTextureType2D, cudaReadModeNormalizedFloat> t, float x, float y, float2 dPdx, float2 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -662,7 +668,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DGrad(
 
 // tex1DLayeredGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayeredGrad(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer, float dPdx, float dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayeredGrad(texture<T, cudaTextureType1DLayered, cudaReadModeElementType> t, float x, int layer, float dPdx, float dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -672,7 +678,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex1DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayeredGrad(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer, float dPdx, float dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayeredGrad(texture<T, cudaTextureType1DLayered, cudaReadModeNormalizedFloat> t, float x, int layer, float dPdx, float dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -684,7 +690,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex1DLayer
 
 // tex2DLayeredGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayeredGrad(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer, float2 dPdx, float2 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayeredGrad(texture<T, cudaTextureType2DLayered, cudaReadModeElementType> t, float x, float y, int layer, float2 dPdx, float2 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -694,7 +700,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex2DLayer
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayeredGrad(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer, float2 dPdx, float2 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayeredGrad(texture<T, cudaTextureType2DLayered, cudaReadModeNormalizedFloat> t, float x, float y, int layer, float2 dPdx, float2 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -706,7 +712,7 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex2DLayer
 
 // tex3DGrad
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DGrad(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z, float4 dPdx, float4 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DGrad(texture<T, cudaTextureType3D, cudaReadModeElementType> t, float x, float y, float z, float4 dPdx, float4 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   typename __nv_tex_rmet_ret<T>::type temp;
@@ -716,7 +722,7 @@ static __forceinline__ __device__ typename __nv_tex_rmet_ret<T>::type tex3DGrad(
 }
 
 template <typename T>
-static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DGrad(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z, float4 dPdx, float4 dPdy)
+static __DEPRECATED__ __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DGrad(texture<T, cudaTextureType3D, cudaReadModeNormalizedFloat> t, float x, float y, float z, float4 dPdx, float4 dPdy)
 { 
 #ifdef __CUDA_ARCH__
   T type_dummy;
@@ -726,6 +732,8 @@ static __forceinline__ __device__ typename __nv_tex_rmnf_ret<T>::type tex3DGrad(
 #endif /* __CUDA_ARCH__ */ 
 }
 
+#undef __DEPRECATED__
+
 #endif /* __cplusplus && __CUDACC__ */
 
 #endif /* !__TEXTURE_FETCH_FUNCTIONS_H__ */
