@@ -2,7 +2,7 @@
 #define NVPERF_TARGET_H
 
 /*
- * Copyright 2014-2020  NVIDIA Corporation.  All rights reserved.
+ * Copyright 2014-2021  NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO USER:
  *
@@ -40,6 +40,7 @@
 
 #include <stddef.h>
 #include <stdint.h>
+#include "nvperf_common.h"
 
 #if defined(__GNUC__) && defined(NVPA_SHARED_LIB)
     #pragma GCC visibility push(default)
@@ -60,237 +61,6 @@ extern "C" {
  *  @file   nvperf_target.h
  */
 
-/***************************************************************************//**
- *  @name   Common Types
- *  @{
- */
-
-#ifndef NVPERF_NVPA_STATUS_DEFINED
-#define NVPERF_NVPA_STATUS_DEFINED
-
-    /// Error codes.
-    typedef enum NVPA_Status
-    {
-        /// Success
-        NVPA_STATUS_SUCCESS = 0,
-        /// Generic error.
-        NVPA_STATUS_ERROR = 1,
-        /// Internal error.  Please file a bug!
-        NVPA_STATUS_INTERNAL_ERROR = 2,
-        /// NVPA_Init() has not been called yet.
-        NVPA_STATUS_NOT_INITIALIZED = 3,
-        /// The NvPerfAPI DLL/DSO could not be loaded during init.
-        NVPA_STATUS_NOT_LOADED = 4,
-        /// The function was not found in this version of the NvPerfAPI DLL/DSO.
-        NVPA_STATUS_FUNCTION_NOT_FOUND = 5,
-        /// The request is intentionally not supported by NvPerfAPI.
-        NVPA_STATUS_NOT_SUPPORTED = 6,
-        /// The request is not implemented by this version of NvPerfAPI.
-        NVPA_STATUS_NOT_IMPLEMENTED = 7,
-        /// Invalid argument.
-        NVPA_STATUS_INVALID_ARGUMENT = 8,
-        /// A MetricId argument does not belong to the specified NVPA_Activity or NVPA_Config.
-        NVPA_STATUS_INVALID_METRIC_ID = 9,
-        /// No driver has been loaded via NVPA_*_LoadDriver().
-        NVPA_STATUS_DRIVER_NOT_LOADED = 10,
-        /// Failed memory allocation.
-        NVPA_STATUS_OUT_OF_MEMORY = 11,
-        /// The request could not be fulfilled due to the state of the current thread.
-        NVPA_STATUS_INVALID_THREAD_STATE = 12,
-        /// Allocation of context object failed.
-        NVPA_STATUS_FAILED_CONTEXT_ALLOC = 13,
-        /// The specified GPU is not supported.
-        NVPA_STATUS_UNSUPPORTED_GPU = 14,
-        /// The installed NVIDIA driver is too old.
-        NVPA_STATUS_INSUFFICIENT_DRIVER_VERSION = 15,
-        /// Graphics object has not been registered via NVPA_Register*().
-        NVPA_STATUS_OBJECT_NOT_REGISTERED = 16,
-        /// The operation failed due to a security check.
-        NVPA_STATUS_INSUFFICIENT_PRIVILEGE = 17,
-        /// The request could not be fulfilled due to the state of the context.
-        NVPA_STATUS_INVALID_CONTEXT_STATE = 18,
-        /// The request could not be fulfilled due to the state of the object.
-        NVPA_STATUS_INVALID_OBJECT_STATE = 19,
-        /// The request could not be fulfilled because a system resource is already in use.
-        NVPA_STATUS_RESOURCE_UNAVAILABLE = 20,
-        /// The NVPA_*_LoadDriver() is called after the context, command queue or device is created.
-        NVPA_STATUS_DRIVER_LOADED_TOO_LATE = 21,
-        /// The provided buffer is not large enough.
-        NVPA_STATUS_INSUFFICIENT_SPACE = 22,
-        /// The API object passed to NVPA_[API]_BeginPass/NVPA_[API]_EndPass and
-        /// NVPA_[API]_PushRange/NVPA_[API]_PopRange does not match with the NVPA_[API]_BeginSession.
-        NVPA_STATUS_OBJECT_MISMATCH = 23,
-        /// Virtualized GPU (vGPU) is not supported.
-        NVPA_STATUS_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 24,
-        NVPA_STATUS__COUNT
-    } NVPA_Status;
-
-
-#endif // NVPERF_NVPA_STATUS_DEFINED
-
-
-#ifndef NVPERF_NVPA_ACTIVITY_KIND_DEFINED
-#define NVPERF_NVPA_ACTIVITY_KIND_DEFINED
-
-    /// The configuration's activity-kind dictates which types of data may be collected.
-    typedef enum NVPA_ActivityKind
-    {
-        /// Invalid value.
-        NVPA_ACTIVITY_KIND_INVALID = 0,
-        /// A workload-centric activity for serialized and pipelined collection.
-        /// 
-        /// Profiler is capable of collecting both serialized and pipelined metrics.  The library introduces any
-        /// synchronization required to collect serialized metrics.
-        NVPA_ACTIVITY_KIND_PROFILER,
-        /// A realtime activity for sampling counters from the CPU or GPU.
-        NVPA_ACTIVITY_KIND_REALTIME_SAMPLED,
-        /// A realtime activity for profiling counters from the CPU or GPU without CPU/GPU synchronizations.
-        NVPA_ACTIVITY_KIND_REALTIME_PROFILER,
-        NVPA_ACTIVITY_KIND__COUNT
-    } NVPA_ActivityKind;
-
-
-#endif // NVPERF_NVPA_ACTIVITY_KIND_DEFINED
-
-
-#ifndef NVPERF_NVPA_BOOL_DEFINED
-#define NVPERF_NVPA_BOOL_DEFINED
-    /// The type used for boolean values.
-    typedef uint8_t NVPA_Bool;
-#endif // NVPERF_NVPA_BOOL_DEFINED
-
-#ifndef NVPA_STRUCT_SIZE
-#define NVPA_STRUCT_SIZE(type_, lastfield_)                     (offsetof(type_, lastfield_) + sizeof(((type_*)0)->lastfield_))
-#endif // NVPA_STRUCT_SIZE
-
-
-#ifndef NVPERF_NVPA_GETPROCADDRESS_DEFINED
-#define NVPERF_NVPA_GETPROCADDRESS_DEFINED
-
-typedef NVPA_Status (*NVPA_GenericFn)(void);
-
-
-    /// 
-    /// Gets the address of a PerfWorks API function.
-    /// 
-    /// \return A function pointer to the function, or NULL if the function is not available.
-    /// 
-    /// \param pFunctionName [in] Name of the function to retrieve.
-    NVPA_GenericFn NVPA_GetProcAddress(const char* pFunctionName);
-
-#endif
-
-#ifndef NVPERF_NVPW_SETLIBRARYLOADPATHS_DEFINED
-#define NVPERF_NVPW_SETLIBRARYLOADPATHS_DEFINED
-
-
-    typedef struct NVPW_SetLibraryLoadPaths_Params
-    {
-        /// [in]
-        size_t structSize;
-        /// [in] assign to NULL
-        void* pPriv;
-        /// [in] number of paths in ppPaths
-        size_t numPaths;
-        /// [in] array of null-terminated paths
-        const char** ppPaths;
-    } NVPW_SetLibraryLoadPaths_Params;
-#define NVPW_SetLibraryLoadPaths_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_SetLibraryLoadPaths_Params, ppPaths)
-
-    /// Sets library search path for \ref NVPA_InitializeHost() and \ref NVPA_InitializeTarget().
-    /// \ref NVPA_InitializeHost() and \ref NVPA_InitializeTarget load the PerfWorks DLL/DSO.  This function sets
-    /// ordered paths that will be searched with the LoadLibrary() or dlopen() call.
-    /// If load paths are set by this function, the default set of load paths
-    /// will not be attempted.
-    /// Each path must point at a directory (not a file name).
-    /// This function is not thread-safe.
-    /// Example Usage:
-    /// \code
-    ///     const char* paths[] = {
-    ///         "path1", "path2", etc
-    ///     };
-    ///     NVPW_SetLibraryLoadPaths_Params params{NVPW_SetLibraryLoadPaths_Params_STRUCT_SIZE};
-    ///     params.numPaths = sizeof(paths)/sizeof(paths[0]);
-    ///     params.ppPaths = paths;
-    ///     NVPW_SetLibraryLoadPaths(&params);
-    ///     NVPA_InitializeHost();
-    ///     params.numPaths = 0;
-    ///     params.ppPaths = NULL;
-    ///     NVPW_SetLibraryLoadPaths(&params);
-    /// \endcode
-    NVPA_Status NVPW_SetLibraryLoadPaths(NVPW_SetLibraryLoadPaths_Params* pParams);
-
-    typedef struct NVPW_SetLibraryLoadPathsW_Params
-    {
-        /// [in]
-        size_t structSize;
-        /// [in] assign to NULL
-        void* pPriv;
-        /// [in] number of paths in ppwPaths
-        size_t numPaths;
-        /// [in] array of null-terminated paths
-        const wchar_t** ppwPaths;
-    } NVPW_SetLibraryLoadPathsW_Params;
-#define NVPW_SetLibraryLoadPathsW_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_SetLibraryLoadPathsW_Params, ppwPaths)
-
-    /// Sets library search path for \ref NVPA_InitializeHost() and \ref NVPA_InitializeTarget().
-    /// \ref NVPA_InitializeHost() and \ref NVPA_InitializeTarget load the PerfWorks DLL/DSO.  This function sets
-    /// ordered paths that will be searched with the LoadLibrary() or dlopen() call.
-    /// If load paths are set by this function, the default set of load paths
-    /// will not be attempted.
-    /// Each path must point at a directory (not a file name).
-    /// This function is not thread-safe.
-    /// Example Usage:
-    /// \code
-    ///     const wchar_t* wpaths[] = {
-    ///         L"path1", L"path2", etc
-    ///     };
-    ///     NVPW_SetLibraryLoadPathsW_Params params{NVPW_SetLibraryLoadPathsW_Params_STRUCT_SIZE};
-    ///     params.numPaths = sizeof(wpaths)/sizeof(wpaths[0]);
-    ///     params.ppwPaths = wpaths;
-    ///     NVPW_SetLibraryLoadPathsW(&params);
-    ///     NVPA_InitializeHost();
-    ///     params.numPaths = 0;
-    ///     params.ppwPaths = NULL;
-    ///     NVPW_SetLibraryLoadPathsW(&params);
-    /// \endcode
-    NVPA_Status NVPW_SetLibraryLoadPathsW(NVPW_SetLibraryLoadPathsW_Params* pParams);
-
-#endif
-
-/**
- *  @}
- ******************************************************************************/
- 
-    /// \deprecated Use of this function is discouraged. Prefer \ref NVPW_InitializeTarget instead.
-    NVPA_Status NVPA_InitializeTarget(void);
-
-
-    // Device enumeration functions must be preceded by NVPA_<API>_LoadDriver(); any API is fine.
-
-
-    /// \deprecated Use of this function is discouraged. Prefer \ref NVPW_GetDeviceCount instead.
-    NVPA_Status NVPA_GetDeviceCount(size_t* pNumDevices);
-
-    /// \deprecated Use of this function is discouraged. Prefer \ref NVPW_Device_GetNames instead.
-    NVPA_Status NVPA_Device_GetNames(
-        size_t deviceIndex,
-        const char** ppDeviceName,
-        const char** ppChipName);
-
-    /// \deprecated Use of this function is discouraged. Prefer \ref NVPW_CounterData_GetNumRanges instead.
-    NVPA_Status NVPA_CounterData_GetNumRanges(
-        const uint8_t* pCounterDataImage,
-        size_t* pNumRanges);
-
-    /// \deprecated Use of this function is discouraged. Prefer \ref NVPW_CounterData_GetRangeDescriptions instead.
-    NVPA_Status NVPA_CounterData_GetRangeDescriptions(
-        const uint8_t* pCounterDataImage,
-        size_t rangeIndex,
-        size_t numDescriptions,
-        const char** ppDescriptions,
-        size_t* pNumDescriptions);
-
 #ifndef NVPW_GPU_ARCHITECTURE_SUPPORT_LEVEL_DEFINED
 #define NVPW_GPU_ARCHITECTURE_SUPPORT_LEVEL_DEFINED
     /// GPU architecture support level
@@ -314,82 +84,41 @@ typedef NVPA_Status (*NVPA_GenericFn)(void);
     } NVPW_SliSupportLevel;
 #endif //NVPW_SLI_SUPPORT_LEVEL_DEFINED
 
-
-    #define NVPW_FIELD_EXISTS(pParams_, name_) \
-        ((pParams_)->structSize >= (const size_t)((const uint8_t*)(&(pParams_)->name_) + sizeof(pParams_)->name_ - (const uint8_t*)(pParams_)))
-    
-
-
-#ifndef NVPERF_PERIODICSAMPLER_TYPES
-#define NVPERF_PERIODICSAMPLER_TYPES
-
-
-#ifndef NVPW_TRIGGER_SOURCE_DEFINED
-#define NVPW_TRIGGER_SOURCE_DEFINED
-    typedef enum NVPW_PeriodicSampler_TriggerSource
+#ifndef NVPW_VGPU_SUPPORT_LEVEL_DEFINED
+#define NVPW_VGPU_SUPPORT_LEVEL_DEFINED
+    /// Virtualized GPU configuration support level
+    typedef enum NVPW_VGpuSupportLevel
     {
-        /// The trigger is based off of system calls.
-        NVPW_TRIGGER_SOURCE_CPU_SYSCALL = 1,
-        /// The trigger is based off of the SYSCLK interval, note SYS frequency by default is variable.
-        NVPW_TRIGGER_SOURCE_GPU_SYSCLK_INTERVAL = 2,
-        /// The trigger is based off of a fixed frequenncy source.
-        NVPW_TRIGGER_SOURCE_GPU_TIME_INTERVAL = 4,
-        /// GR pushbuffer trigger that can come from other processes.
-        NVPW_TRIGGER_SOURCE_GPU_ENGINE_TRIGGER = 8,
-        NVPW_TRIGGER_SOURCE__COUNT
-    } NVPW_PeriodicSampler_TriggerSource;
-#endif //NVPW_TRIGGER_SOURCE_DEFINED
+        NVPW_VGPU_SUPPORT_LEVEL_UNKNOWN = 0,
+        NVPW_VGPU_SUPPORT_LEVEL_UNSUPPORTED,
+        /// Supported but not allowed by system admin.
+        NVPW_VGPU_SUPPORT_LEVEL_SUPPORTED_DISALLOWED,
+        NVPW_VGPU_SUPPORT_LEVEL_SUPPORTED_ALLOWED,
+        NVPW_VGPU_SUPPORT_LEVEL_SUPPORTED_NON_VGPU_CONFIGURATION
+    } NVPW_VGpuSupportLevel;
+#endif //NVPW_VGPU_SUPPORT_LEVEL_DEFINED
 
-    typedef struct NVPW_PeriodicSampler_SampleOptions
+#ifndef NVPW_CONF_COMPUTE_SUPPORT_LEVEL_DEFINED
+#define NVPW_CONF_COMPUTE_SUPPORT_LEVEL_DEFINED
+    /// Confidential Compute mode support level
+    typedef enum NVPW_ConfidentialComputeSupportLevel
     {
-        /// [in]
-        size_t structSize;
-        /// [in] assign to NULL
-        void* pPriv;
-        /// maximum number of StartSampling/StopSampling pairs in a single frame
-        size_t maxSamplingRangesPerFrame;
-        /// maximum number of user delimiters that can be inserted via InsertDelimiter in a single frame
-        size_t maxUserDelimitersPerFrame;
-        /// maximum number of samples in a single frame
-        size_t maxSamplesPerFrame;
-        union
-        {
-            /// In SYSCLK cycles. Only applies to 'NVPW_TRIGGER_SOURCE_GPU_SYSCLK_INTERVAL'.
-            size_t sampleIntervalCycles;
-            /// Only applies to 'NVPW_TRIGGER_SOURCE_GPU_TIME_INTERVAL'.
-            size_t sampleIntervalNanoseconds;
-        };
-        /// average length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
-        /// character. Must be less than or equal to maxDelimiterNameLength.
-        size_t avgDelimiterNameLength;
-        /// maximum length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
-        /// character. Must be less than 256.
-        size_t maxDelimiterNameLength;
-        /// set to the maximum possible number of unread passes, must be at least 2
-        size_t numTraceBuffers;
-        uint32_t triggerSource;
-    } NVPW_PeriodicSampler_SampleOptions;
-#define NVPW_PeriodicSampler_SampleOptions_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_PeriodicSampler_SampleOptions, triggerSource)
+        NVPW_CONF_COMPUTE_SUPPORT_LEVEL_UNKNOWN = 0,
+        NVPW_CONF_COMPUTE_SUPPORT_LEVEL_UNSUPPORTED,
+        NVPW_CONF_COMPUTE_SUPPORT_LEVEL_SUPPORTED_NON_CONF_COMPUTE_CONFIGURATION
+    } NVPW_ConfidentialComputeSupportLevel;
+#endif //NVPW_CONF_COMPUTE_SUPPORT_LEVEL_DEFINED
 
-    typedef struct NVPW_PeriodicSampler_CounterDataImageOptions
+#ifndef NVPW_CMP_SUPPORT_LEVEL_DEFINED
+#define NVPW_CMP_SUPPORT_LEVEL_DEFINED
+    /// CMP support level
+    typedef enum NVPW_CmpSupportLevel
     {
-        /// [in]
-        size_t structSize;
-        /// The CounterDataPrefix generated from e.g.    nvperf2 initdata   or
-        /// NVPA_CounterDataBuilder_GetCounterDataPrefix().  Must be align(8).
-        const uint8_t* pCounterDataPrefix;
-        size_t counterDataPrefixSize;
-        /// maximum length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
-        /// character. Must be less than 256.
-        size_t maxDelimiterNameLength;
-        /// maximum number of samples in a single frame
-        size_t maxSamplesPerFrame;
-    } NVPW_PeriodicSampler_CounterDataImageOptions;
-#define NVPW_PeriodicSampler_CounterDataImageOptions_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_PeriodicSampler_CounterDataImageOptions, maxSamplesPerFrame)
-
-
-#endif //NVPERF_PERIODICSAMPLER_TYPES
-
+        NVPW_CMP_SUPPORT_LEVEL_UNKNOWN = 0,
+        NVPW_CMP_SUPPORT_LEVEL_UNSUPPORTED,
+        NVPW_CMP_SUPPORT_LEVEL_SUPPORTED_NON_CMP_CONFIGURATON
+    } NVPW_CmpSupportLevel;
+#endif //NVPW_CMP_SUPPORT_LEVEL_DEFINED
 
     typedef struct NVPW_InitializeTarget_Params
     {
@@ -509,6 +238,23 @@ typedef NVPA_Status (*NVPA_GenericFn)(void);
 
     NVPA_Status NVPW_CounterData_GetNumRanges(NVPW_CounterData_GetNumRanges_Params* pParams);
 
+    typedef struct NVPW_CounterData_GetChipName_Params
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        /// [in]
+        const uint8_t* pCounterDataImage;
+        /// [in]
+        size_t counterDataImageSize;
+        /// [out]
+        const char* pChipName;
+    } NVPW_CounterData_GetChipName_Params;
+#define NVPW_CounterData_GetChipName_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_CounterData_GetChipName_Params, pChipName)
+
+    NVPA_Status NVPW_CounterData_GetChipName(NVPW_CounterData_GetChipName_Params* pParams);
+
     typedef struct NVPW_Config_GetNumPasses_Params
     {
         /// [in]
@@ -527,6 +273,145 @@ typedef NVPA_Status (*NVPA_GenericFn)(void);
     /// Total num passes = numPipelinedPasses + numIsolatedPasses * numNestingLevels
     NVPA_Status NVPW_Config_GetNumPasses(NVPW_Config_GetNumPasses_Params* pParams);
 
+    typedef struct NVPW_Config_GetNumPasses_V2_Params
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        /// [in]
+        const uint8_t* pConfig;
+        /// [out]
+        size_t numPasses;
+    } NVPW_Config_GetNumPasses_V2_Params;
+#define NVPW_Config_GetNumPasses_V2_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_Config_GetNumPasses_V2_Params, numPasses)
+
+    /// Total num passes = numPasses * numNestingLevels
+    NVPA_Status NVPW_Config_GetNumPasses_V2(NVPW_Config_GetNumPasses_V2_Params* pParams);
+
+#define NVPW_API_SET_CUDA_PROFILER             0x18209d0775b2f89dULL
+
+#define NVPW_API_SET_D3D11_PROFILER            0xca55c6738445db2bULL
+
+#define NVPW_API_SET_D3D12_PERIODICSAMPLER     0xe81a80d42a3f007fULL
+
+#define NVPW_API_SET_D3D12_PROFILER            0xc0c2d46dd7c7ad78ULL
+
+#define NVPW_API_SET_EGL_PROFILER              0x3c3747dae1f9565cULL
+
+#define NVPW_API_SET_METRICSCONTEXT            0x7c8579f6f2144beaULL
+
+#define NVPW_API_SET_METRICSEVALUATOR          0x0368a8768d811af9ULL
+
+#define NVPW_API_SET_METRICS_GA100_COMP        0x16b7d8c20d8b4915ULL
+
+#define NVPW_API_SET_METRICS_GA100_GRFX        0xc94eaabec04a94faULL
+
+#define NVPW_API_SET_METRICS_GA10X_COMP        0xb5d6391c2e299ab5ULL
+
+#define NVPW_API_SET_METRICS_GA10X_GRFX        0x6ebc121178b5ce0bULL
+
+#define NVPW_API_SET_METRICS_GV100_COMP        0x863705cc57919f72ULL
+
+#define NVPW_API_SET_METRICS_GV100_GRFX        0x9900da75d164fecfULL
+
+#define NVPW_API_SET_METRICS_GV11B_COMP        0xd3f79a859235848fULL
+
+#define NVPW_API_SET_METRICS_GV11B_GRFX        0xeb8e26220106e227ULL
+
+#define NVPW_API_SET_METRICS_TU10X_COMP        0x70f40be0afd35da8ULL
+
+#define NVPW_API_SET_METRICS_TU10X_GRFX        0xdf219cb838db6968ULL
+
+#define NVPW_API_SET_METRICS_TU11X_COMP        0xeb0069d7d0956678ULL
+
+#define NVPW_API_SET_METRICS_TU11X_GRFX        0x0977d9342bd62743ULL
+
+#define NVPW_API_SET_OPENGL_PROFILER           0xe4cd9ea40f2ee777ULL
+
+#define NVPW_API_SET_VULKAN_PERIODICSAMPLER    0xdc999b5368e416a0ULL
+
+#define NVPW_API_SET_VULKAN_PROFILER           0x8c56b6a03d779689ULL
+
+    typedef struct NVPW_QueryVersionNumber_Params
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        /// [in]
+        uint64_t apiSet;
+        /// [out]
+        uint32_t major;
+        /// [out]
+        uint32_t minor;
+        /// [out]
+        uint32_t patch;
+        /// [out]
+        uint32_t relMajor;
+        /// [out]
+        uint32_t relMinor;
+        /// [out]
+        uint32_t relPatch;
+    } NVPW_QueryVersionNumber_Params;
+#define NVPW_QueryVersionNumber_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_QueryVersionNumber_Params, relPatch)
+
+    /// Query version number of an API set
+    NVPA_Status NVPW_QueryVersionNumber(NVPW_QueryVersionNumber_Params* pParams);
+
+    typedef enum NVPW_Device_ClockStatus
+    {
+        /// clock status is unknown
+        NVPW_DEVICE_CLOCK_STATUS_UNKNOWN,
+        /// clocks are locked to rated tdp values
+        NVPW_DEVICE_CLOCK_STATUS_LOCKED_TO_RATED_TDP,
+        /// clocks are not locked and can boost above rated tdp
+        NVPW_DEVICE_CLOCK_STATUS_BOOST_ENABLED,
+        /// clocks are not locked and will not go above rated tdp
+        NVPW_DEVICE_CLOCK_STATUS_BOOST_DISABLED,
+        NVPW_DEVICE_CLOCK_STATUS__COUNT
+    } NVPW_Device_ClockStatus;
+
+    typedef struct NVPW_Device_GetClockStatus_Params
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        size_t deviceIndex;
+        /// [in]
+        NVPW_Device_ClockStatus clockStatus;
+    } NVPW_Device_GetClockStatus_Params;
+#define NVPW_Device_GetClockStatus_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_Device_GetClockStatus_Params, clockStatus)
+
+    NVPA_Status NVPW_Device_GetClockStatus(NVPW_Device_GetClockStatus_Params* pParams);
+
+    typedef enum NVPW_Device_ClockSetting
+    {
+        /// invalid op, specify valid clocks operation during profiling
+        NVPW_DEVICE_CLOCK_SETTING_INVALID,
+        /// default to driver/application config (normally unlocked and not boosted, but could be unlocked boosted, or
+        /// locked to rated TDP)
+        NVPW_DEVICE_CLOCK_SETTING_DEFAULT,
+        /// lock clocks at rated tdp base values
+        NVPW_DEVICE_CLOCK_SETTING_LOCK_TO_RATED_TDP,
+        NVPW_DEVICE_CLOCK_SETTING__COUNT
+    } NVPW_Device_ClockSetting;
+
+    typedef struct NVPW_Device_SetClockSetting_Params
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        size_t deviceIndex;
+        /// [in]
+        NVPW_Device_ClockSetting clockSetting;
+    } NVPW_Device_SetClockSetting_Params;
+#define NVPW_Device_SetClockSetting_Params_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_Device_SetClockSetting_Params, clockSetting)
+
+    NVPA_Status NVPW_Device_SetClockSetting(NVPW_Device_SetClockSetting_Params* pParams);
+
     typedef struct NVPW_CounterData_GetRangeDescriptions_Params
     {
         /// [in]
@@ -559,6 +444,68 @@ typedef NVPA_Status (*NVPA_GenericFn)(void);
 
     NVPA_Status NVPW_Profiler_CounterData_GetRangeDescriptions(NVPW_Profiler_CounterData_GetRangeDescriptions_Params* pParams);
 
+#ifndef NVPW_TRIGGER_SOURCE_DEFINED
+#define NVPW_TRIGGER_SOURCE_DEFINED
+    typedef enum NVPW_PeriodicSampler_TriggerSource
+    {
+        /// The trigger is based off of system calls.
+        NVPW_TRIGGER_SOURCE_CPU_SYSCALL = 1,
+        /// The trigger is based off of the SYSCLK interval, note SYS frequency by default is variable.
+        NVPW_TRIGGER_SOURCE_GPU_SYSCLK_INTERVAL = 2,
+        /// The trigger is based off of a fixed frequency source.
+        NVPW_TRIGGER_SOURCE_GPU_TIME_INTERVAL = 4,
+        /// GR pushbuffer trigger that can come from this or other processes.
+        NVPW_TRIGGER_SOURCE_GPU_ENGINE_TRIGGER = 8,
+        NVPW_TRIGGER_SOURCE__COUNT
+    } NVPW_PeriodicSampler_TriggerSource;
+#endif //NVPW_TRIGGER_SOURCE_DEFINED
+
+    typedef struct NVPW_PeriodicSampler_SampleOptions
+    {
+        /// [in]
+        size_t structSize;
+        /// [in] assign to NULL
+        void* pPriv;
+        /// maximum number of StartSampling/StopSampling pairs in a single frame
+        size_t maxSamplingRangesPerFrame;
+        /// maximum number of user delimiters that can be inserted via InsertDelimiter in a single frame
+        size_t maxUserDelimitersPerFrame;
+        /// maximum number of samples in a single frame
+        size_t maxSamplesPerFrame;
+        union
+        {
+            /// In SYSCLK cycles. Only applies to 'NVPW_TRIGGER_SOURCE_GPU_SYSCLK_INTERVAL'.
+            size_t sampleIntervalCycles;
+            /// Only applies to 'NVPW_TRIGGER_SOURCE_GPU_TIME_INTERVAL'.
+            size_t sampleIntervalNanoseconds;
+        };
+        /// average length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
+        /// character. Must be less than or equal to maxDelimiterNameLength.
+        size_t avgDelimiterNameLength;
+        /// maximum length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
+        /// character. Must be less than 256.
+        size_t maxDelimiterNameLength;
+        /// set to the maximum possible number of unread passes, must be at least 2
+        size_t numTraceBuffers;
+        uint32_t triggerSource;
+    } NVPW_PeriodicSampler_SampleOptions;
+#define NVPW_PeriodicSampler_SampleOptions_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_PeriodicSampler_SampleOptions, triggerSource)
+
+    typedef struct NVPW_PeriodicSampler_CounterDataImageOptions
+    {
+        /// [in]
+        size_t structSize;
+        /// The CounterDataPrefix generated from e.g. NVPW_CounterDataBuilder_GetCounterDataPrefix().  Must be align(8).
+        const uint8_t* pCounterDataPrefix;
+        size_t counterDataPrefixSize;
+        /// maximum length of the delimiters passed to StartSampling/InsertDelimiter, excluding the trailing NULL
+        /// character. Must be less than 256.
+        size_t maxDelimiterNameLength;
+        /// maximum number of samples in a single frame
+        size_t maxSamplesPerFrame;
+    } NVPW_PeriodicSampler_CounterDataImageOptions;
+#define NVPW_PeriodicSampler_CounterDataImageOptions_STRUCT_SIZE NVPA_STRUCT_SIZE(NVPW_PeriodicSampler_CounterDataImageOptions, maxSamplesPerFrame)
+
     typedef struct NVPW_PeriodicSampler_CounterData_DelimiterInfo
     {
         const char* pDelimiterName;
