@@ -81,6 +81,11 @@ typedef struct cudaIpcCloseMemHandle_v4010_params_st {
     void *devPtr;
 } cudaIpcCloseMemHandle_v4010_params;
 
+typedef struct cudaDeviceFlushGPUDirectRDMAWrites_v11030_params_st {
+    enum cudaFlushGPUDirectRDMAWritesTarget target;
+    enum cudaFlushGPUDirectRDMAWritesScope scope;
+} cudaDeviceFlushGPUDirectRDMAWrites_v11030_params;
+
 typedef struct cudaGetErrorName_v6050_params_st {
     cudaError_t error;
 } cudaGetErrorName_v6050_params;
@@ -257,6 +262,22 @@ typedef struct cudaStreamGetCaptureInfo_ptsz_v10010_params_st {
     unsigned long long *pId;
 } cudaStreamGetCaptureInfo_ptsz_v10010_params;
 
+typedef struct cudaStreamGetCaptureInfo_v2_params_st {
+    cudaStream_t stream;
+    enum cudaStreamCaptureStatus *captureStatus_out;
+    unsigned long long *id_out;
+    cudaGraph_t *graph_out;
+    const cudaGraphNode_t **dependencies_out;
+    size_t *numDependencies_out;
+} cudaStreamGetCaptureInfo_v2_params;
+
+typedef struct cudaStreamUpdateCaptureDependencies_v11030_params_st {
+    cudaStream_t stream;
+    cudaGraphNode_t *dependencies;
+    size_t numDependencies;
+    unsigned int flags;
+} cudaStreamUpdateCaptureDependencies_v11030_params;
+
 typedef struct cudaEventCreate_v3020_params_st {
     cudaEvent_t *event;
 } cudaEventCreate_v3020_params;
@@ -1339,6 +1360,48 @@ typedef struct cudaGraphExternalSemaphoresWaitNodeSetParams_v11020_params_st {
     const struct cudaExternalSemaphoreWaitNodeParams *nodeParams;
 } cudaGraphExternalSemaphoresWaitNodeSetParams_v11020_params;
 
+typedef struct cudaGraphAddMemAllocNode_v11040_params_st {
+    cudaGraphNode_t *pGraphNode;
+    cudaGraph_t graph;
+    const cudaGraphNode_t *pDependencies;
+    size_t numDependencies;
+    struct cudaMemAllocNodeParams *nodeParams;
+} cudaGraphAddMemAllocNode_v11040_params;
+
+typedef struct cudaGraphMemAllocNodeGetParams_v11040_params_st {
+    cudaGraphNode_t node;
+    struct cudaMemAllocNodeParams *params_out;
+} cudaGraphMemAllocNodeGetParams_v11040_params;
+
+typedef struct cudaGraphAddMemFreeNode_v11040_params_st {
+    cudaGraphNode_t *pGraphNode;
+    cudaGraph_t graph;
+    const cudaGraphNode_t *pDependencies;
+    size_t numDependencies;
+    void *dptr;
+} cudaGraphAddMemFreeNode_v11040_params;
+
+typedef struct cudaGraphMemFreeNodeGetParams_v11040_params_st {
+    cudaGraphNode_t node;
+    void *dptr_out;
+} cudaGraphMemFreeNodeGetParams_v11040_params;
+
+typedef struct cudaDeviceGraphMemTrim_v11040_params_st {
+    int device;
+} cudaDeviceGraphMemTrim_v11040_params;
+
+typedef struct cudaDeviceGetGraphMemAttribute_v11040_params_st {
+    int device;
+    enum cudaGraphMemAttributeType attr;
+    void *value;
+} cudaDeviceGetGraphMemAttribute_v11040_params;
+
+typedef struct cudaDeviceSetGraphMemAttribute_v11040_params_st {
+    int device;
+    enum cudaGraphMemAttributeType attr;
+    void *value;
+} cudaDeviceSetGraphMemAttribute_v11040_params;
+
 typedef struct cudaGraphClone_v10000_params_st {
     cudaGraph_t *pGraphClone;
     cudaGraph_t originalGraph;
@@ -1412,6 +1475,12 @@ typedef struct cudaGraphInstantiate_v10000_params_st {
     size_t bufferSize;
 } cudaGraphInstantiate_v10000_params;
 
+typedef struct cudaGraphInstantiateWithFlags_v11040_params_st {
+    cudaGraphExec_t *pGraphExec;
+    cudaGraph_t graph;
+    unsigned long long flags;
+} cudaGraphInstantiateWithFlags_v11040_params;
+
 typedef struct cudaGraphExecKernelNodeSetParams_v10010_params_st {
     cudaGraphExec_t hGraphExec;
     cudaGraphNode_t node;
@@ -1520,6 +1589,49 @@ typedef struct cudaGraphDestroy_v10000_params_st {
     cudaGraph_t graph;
 } cudaGraphDestroy_v10000_params;
 
+typedef struct cudaGraphDebugDotPrint_v11030_params_st {
+    cudaGraph_t graph;
+    const char *path;
+    unsigned int flags;
+} cudaGraphDebugDotPrint_v11030_params;
+
+typedef struct cudaUserObjectCreate_v11030_params_st {
+    cudaUserObject_t *object_out;
+    void *ptr;
+    cudaHostFn_t destroy;
+    unsigned int initialRefcount;
+    unsigned int flags;
+} cudaUserObjectCreate_v11030_params;
+
+typedef struct cudaUserObjectRetain_v11030_params_st {
+    cudaUserObject_t object;
+    unsigned int count;
+} cudaUserObjectRetain_v11030_params;
+
+typedef struct cudaUserObjectRelease_v11030_params_st {
+    cudaUserObject_t object;
+    unsigned int count;
+} cudaUserObjectRelease_v11030_params;
+
+typedef struct cudaGraphRetainUserObject_v11030_params_st {
+    cudaGraph_t graph;
+    cudaUserObject_t object;
+    unsigned int count;
+    unsigned int flags;
+} cudaGraphRetainUserObject_v11030_params;
+
+typedef struct cudaGraphReleaseUserObject_v11030_params_st {
+    cudaGraph_t graph;
+    cudaUserObject_t object;
+    unsigned int count;
+} cudaGraphReleaseUserObject_v11030_params;
+
+typedef struct cudaGetDriverEntryPoint_ptsz_v11030_params_st {
+    const char *symbol;
+    void **funcPtr;
+    unsigned long long flags;
+} cudaGetDriverEntryPoint_ptsz_v11030_params;
+
 typedef struct cudaGetFuncBySymbol_v11000_params_st {
     cudaFunction_t *functionPtr;
     const void *symbolPtr;
@@ -1857,6 +1969,13 @@ typedef struct cudaSignalExternalSemaphoresAsync_ptsz_v10000_params_st {
     cudaStream_t stream;
 } cudaSignalExternalSemaphoresAsync_ptsz_v10000_params;
 
+typedef struct cudaSignalExternalSemaphoresAsync_v2_params_st {
+    const cudaExternalSemaphore_t *extSemArray;
+    const struct cudaExternalSemaphoreSignalParams *paramsArray;
+    unsigned int numExtSems;
+    cudaStream_t stream;
+} cudaSignalExternalSemaphoresAsync_v2_params;
+
 typedef struct cudaWaitExternalSemaphoresAsync_v10000_params_st {
     const cudaExternalSemaphore_t *extSemArray;
     const struct cudaExternalSemaphoreWaitParams_v1 *paramsArray;
@@ -1871,6 +1990,13 @@ typedef struct cudaWaitExternalSemaphoresAsync_ptsz_v10000_params_st {
     cudaStream_t stream;
 } cudaWaitExternalSemaphoresAsync_ptsz_v10000_params;
 
+typedef struct cudaWaitExternalSemaphoresAsync_v2_params_st {
+    const cudaExternalSemaphore_t *extSemArray;
+    const struct cudaExternalSemaphoreWaitParams *paramsArray;
+    unsigned int numExtSems;
+    cudaStream_t stream;
+} cudaWaitExternalSemaphoresAsync_v2_params;
+
 typedef struct cudaGraphUpload_v10000_params_st {
     cudaGraphExec_t graphExec;
     cudaStream_t stream;
@@ -1897,11 +2023,27 @@ typedef struct cudaStreamIsCapturing_v10000_params_st {
 } cudaStreamIsCapturing_v10000_params;
 
 typedef struct cudaStreamGetCaptureInfo_v10010_params_st {
-    cudaStream_t hStream;
-    enum cudaStreamCaptureStatus *pCaptureStatus;
-    unsigned long long *pId;
+    cudaStream_t stream;
+    enum cudaStreamCaptureStatus *captureStatus_out;
+    unsigned long long *id_out;
 } cudaStreamGetCaptureInfo_v10010_params;
 
+typedef struct cudaStreamGetCaptureInfo_v2_params_st {
+    cudaStream_t stream;
+    enum cudaStreamCaptureStatus *captureStatus_out;
+    unsigned long long *id_out;
+    cudaGraph_t *graph_out;
+    const cudaGraphNode_t **dependencies_out;
+    size_t *numDependencies_out;
+} cudaStreamGetCaptureInfo_v2_params;
+
+typedef struct cudaStreamUpdateCaptureDependencies_ptsz_v11030_params_st {
+    cudaStream_t stream;
+    cudaGraphNode_t *dependencies;
+    size_t numDependencies;
+    unsigned int flags;
+} cudaStreamUpdateCaptureDependencies_ptsz_v11030_params;
+
 typedef struct cudaStreamCopyAttributes_v11000_params_st {
     cudaStream_t dstStream;
     cudaStream_t srcStream;
@@ -1937,6 +2079,12 @@ typedef struct cudaMallocFromPoolAsync_v11020_params_st {
     cudaStream_t stream;
 } cudaMallocFromPoolAsync_v11020_params;
 
+typedef struct cudaGetDriverEntryPoint_v11030_params_st {
+    const char *symbol;
+    void **funcPtr;
+    unsigned long long flags;
+} cudaGetDriverEntryPoint_v11030_params;
+
 // Parameter trace structures for removed functions 
 
 
