@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2020 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2018-2021 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -243,82 +243,82 @@ typedef enum {
     /**
      * Invalid resource callback ID.
      */
-    SANITIZER_CBID_RESOURCE_INVALID                   = 0,
+    SANITIZER_CBID_RESOURCE_INVALID                       = 0,
 
     /**
      * Driver initialization is finished.
      */
-    SANITIZER_CBID_RESOURCE_INIT_FINISHED             = 1,
+    SANITIZER_CBID_RESOURCE_INIT_FINISHED                 = 1,
 
     /**
      * A new context is about to be created.
      */
-    SANITIZER_CBID_RESOURCE_CONTEXT_CREATION_STARTING = 2,
+    SANITIZER_CBID_RESOURCE_CONTEXT_CREATION_STARTING     = 2,
 
     /**
      * A new context was created.
      */
-    SANITIZER_CBID_RESOURCE_CONTEXT_CREATION_FINISHED = 3,
+    SANITIZER_CBID_RESOURCE_CONTEXT_CREATION_FINISHED     = 3,
 
     /**
      * A context is about to be destroyed.
      */
-    SANITIZER_CBID_RESOURCE_CONTEXT_DESTROY_STARTING  = 4,
+    SANITIZER_CBID_RESOURCE_CONTEXT_DESTROY_STARTING      = 4,
 
     /**
      * A context was destroyed.
      */
-    SANITIZER_CBID_RESOURCE_CONTEXT_DESTROY_FINISHED  = 5,
+    SANITIZER_CBID_RESOURCE_CONTEXT_DESTROY_FINISHED      = 5,
 
     /**
      * A new stream was created.
      */
-    SANITIZER_CBID_RESOURCE_STREAM_CREATED            = 6,
+    SANITIZER_CBID_RESOURCE_STREAM_CREATED                = 6,
 
     /**
      * A stream is about to be destroyed.
      */
-    SANITIZER_CBID_RESOURCE_STREAM_DESTROY_STARTING   = 7,
+    SANITIZER_CBID_RESOURCE_STREAM_DESTROY_STARTING       = 7,
 
     /**
      * A module was loaded.
      */
-    SANITIZER_CBID_RESOURCE_MODULE_LOADED             = 8,
+    SANITIZER_CBID_RESOURCE_MODULE_LOADED                 = 8,
 
     /**
      * A module is about to be unloaded.
      */
-    SANITIZER_CBID_RESOURCE_MODULE_UNLOAD_STARTING    = 9,
+    SANITIZER_CBID_RESOURCE_MODULE_UNLOAD_STARTING        = 9,
 
     /**
      * Device memory was allocated.
      */
-    SANITIZER_CBID_RESOURCE_DEVICE_MEMORY_ALLOC       = 10,
+    SANITIZER_CBID_RESOURCE_DEVICE_MEMORY_ALLOC           = 10,
 
     /**
      * Device memory was freed.
      */
-    SANITIZER_CBID_RESOURCE_DEVICE_MEMORY_FREE        = 11,
+    SANITIZER_CBID_RESOURCE_DEVICE_MEMORY_FREE            = 11,
 
     /**
      * Pinned host memory was allocated.
      */
-    SANITIZER_CBID_RESOURCE_HOST_MEMORY_ALLOC         = 12,
+    SANITIZER_CBID_RESOURCE_HOST_MEMORY_ALLOC             = 12,
 
     /**
      * Pinned host memory was freed.
      */
-    SANITIZER_CBID_RESOURCE_HOST_MEMORY_FREE          = 13,
+    SANITIZER_CBID_RESOURCE_HOST_MEMORY_FREE              = 13,
 
     /**
      * Memory was allocated asynchronously.
      */
-    SANITIZER_CBID_RESOURCE_MEMORY_ALLOC_ASYNC        = 14,
+    SANITIZER_CBID_RESOURCE_MEMORY_ALLOC_ASYNC            = 14,
 
     /**
      * Memory was freed asynchronously.
      */
-    SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC         = 15,
+    SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC             = 15,
 
     /**
      * Memory freed asynchronously was released, only happens if a regular
@@ -326,17 +326,17 @@ typedef enum {
      *
      * See CUDA runtime documentation for cudaFreeAsync
      */
-    SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC_DONE    = 16,
+    SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC_DONE        = 16,
 
     /**
      * A new mempool was created.
      */
-    SANITIZER_CBID_RESOURCE_MEMPOOL_CREATED           = 17,
+    SANITIZER_CBID_RESOURCE_MEMPOOL_CREATED               = 17,
 
     /**
      * A mempool is about to be destroyed.
      */
-    SANITIZER_CBID_RESOURCE_MEMPOOL_DESTROYING        = 18,
+    SANITIZER_CBID_RESOURCE_MEMPOOL_DESTROYING            = 18,
 
     /**
      * A mempool is now accessible from a peer device.
@@ -348,8 +348,18 @@ typedef enum {
      */
     SANITIZER_CBID_RESOURCE_MEMPOOL_PEER_ACCESS_DISABLING = 20,
 
+    /**
+     * A CUDA array was created.
+     */
+    SANITIZER_CBID_RESOURCE_ARRAY_CREATED                 = 21,
+
+    /**
+     * A CUDA array was destroyed.
+     */
+    SANITIZER_CBID_RESOURCE_ARRAY_DESTROYED               = 22,
+
     SANITIZER_CBID_RESOURCE_SIZE,
-    SANITIZER_CBID_RESOURCE_FORCE_INT                 = 0x7fffffff
+    SANITIZER_CBID_RESOURCE_FORCE_INT                     = 0x7fffffff
 } Sanitizer_CallbackIdResource;
 
 /**
@@ -531,17 +541,17 @@ typedef enum {
     SANITIZER_MEMORY_PERMISSION_WRITE          = 0x2,
 
     /**
-     * Specifies that the allocation is readable and
-     * writable (shortcut for READ | WRITE).
+     * Specifies that the allocation is readable/writable with atomic
+     * operations.
      */
-    SANITIZER_MEMORY_PERMISSION_READ_AND_WRITE = 0x3,
+    SANITIZER_MEMORY_PERMISSION_ATOMIC         = 0x4,
 
     /**
-     * Specifies that the allocation is executable.
+     * Specifies that the allocation has all permissions.
      */
-    SANITIZER_MEMORY_PERMISSION_EXECUTE        = 0x4,
+    SANITIZER_MEMORY_PERMISSION_ALL            = 0x7,
 
-    SANITIZER_MEMORY_PERMISSION_FORCE_INT       = 0x7fffffff
+    SANITIZER_MEMORY_PERMISSION_FORCE_INT      = 0x7fffffff
 } Sanitizer_ResourceMemoryPermissions;
 
 /**
@@ -591,8 +601,9 @@ typedef enum {
  * SANITIZER_CBID_RESOURCE_DEVICE_MEMORY_FREE,
  * SANITIZER_CBID_RESOURCE_HOST_MEMORY_ALLOC,
  * SANITIZER_CBID_RESOURCE_HOST_MEMORY_FREE,
- * SANITIZER_CBID_RESOURCE_SURFACE_OBJECT_CREATED or
- * SANITIZER_CBID_RESOURCE_SURFACE_OBJECT_DESTROY.
+ * SANITIZER_CBID_RESOURCE_MEMORY_ALLOC_ASYNC,
+ * SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC or
+ * SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC_DONE.
  * The callback data is only valid within the invocation of the
  * callback function that is passed the data. If you need to
  * retain some data for use outside of the callback, you must
@@ -617,10 +628,10 @@ typedef struct {
     CUcontext context;
 
     /**
-     * Id of the device where the allocation is being created.
+     * Device where the allocation is being created.
      * Available for all cbid with a driver version of 455 or newer.
      */
-    uint32_t deviceId;
+    CUdevice device;
 
     /**
      * Public handle for the stream.
@@ -657,6 +668,12 @@ typedef struct {
      * Visibility of the allocation.
      */
     Sanitizer_MemoryVisibility visibility;
+
+    /**
+     * Source device of this allocation (different from device if
+     * SANITIZER_MEMORY_FLAG_PEER is set).
+     */
+    CUdevice sourceDevice;
 } Sanitizer_ResourceMemoryData;
 
 /**
@@ -694,6 +711,41 @@ typedef struct {
     CUdevice peerDevice;
 } Sanitizer_ResourceMempoolData;
 
+/**
+ *
+ * \brief Data passed into a CUDA array callback function.
+ *
+ * Data passed into a  CUDA array callback function as the
+ * \p cbdata argument to \ref Sanitizer_CallbackFunc. The
+ * \p cbdata will be this type for \p domain equal tp
+ * SANITIZER_CB_DOMAIN_RESOURCE and \p cbid equal to
+ * SANITIZER_CBID_RESOURCE_ARRAY_CREATED or
+ * SANITIZER_CBID_RESOURCE_ARRAY_DESTROYED.
+ * The callback data is only valid within the invocation of the
+ * callback function that is passed the data. If you need to
+ * retain some data for use outside of the callback, you must
+ * make a copy of it.
+ */
+typedef struct {
+    /**
+     * The context containing the array being created or
+     * destroyed.
+     */
+    CUcontext context;
+
+    /**
+     * The CUDA array being created or destroyed.
+     */
+    CUarray hArray;
+
+    /**
+     * The CUDA array size.
+     */
+    uint64_t width;
+    uint64_t height;
+    uint64_t depth;
+} Sanitizer_ResourceArrayData;
+
 /**
  * \brief Callback IDs for synchronization domain.
  *
@@ -857,7 +909,7 @@ typedef struct {
     /**
      * The device where the grid is launched
      */
-    uint32_t deviceId;
+    CUdevice device;
 } Sanitizer_LaunchData;
 
 /**
@@ -1312,10 +1364,27 @@ typedef struct {
     CUgraphNodeType nodeType;
 
     /**
-     * Launch data for this node launch.
-     * This is only valid if nodeType is CU_GRAPH_NODE_TYPE_KERNEL.
+     * Data for this node launch.
+     *
      */
-    Sanitizer_LaunchData launchData;
+
+    union {
+        /**
+         * This is only valid if nodeType is CU_GRAPH_NODE_TYPE_KERNEL.
+         */
+        Sanitizer_LaunchData launchData;
+
+        /**
+         * This is only valid if nodeType is CU_GRAPH_NODE_TYPE_MEM_ALLOC.
+         */
+        Sanitizer_ResourceMemoryData memAllocData;
+
+        /**
+         * The freed device pointer
+         * This is only valid if nodeType is CU_GRAPH_NODE_TYPE_MEM_FREE.
+         */
+        uint64_t memFreeAddress;
+    };
 
 } Sanitizer_GraphNodeLaunchData;
 
