@@ -397,7 +397,28 @@ typedef enum SANITIZER_CBID_RUNTIME_API_enum {
     SANITIZER_CBID_RUNTIME_API_cudaGraphExternalSemaphoresWaitNodeSetParams                = 402,
     SANITIZER_CBID_RUNTIME_API_cudaGraphExecExternalSemaphoresSignalNodeSetParams          = 403,
     SANITIZER_CBID_RUNTIME_API_cudaGraphExecExternalSemaphoresWaitNodeSetParams            = 404,
-    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 405,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceFlushGPUDirectRDMAWrites                          = 405,
+    SANITIZER_CBID_RUNTIME_API_cudaGetDriverEntryPoint                                     = 406,
+    SANITIZER_CBID_RUNTIME_API_cudaGetDriverEntryPoint_ptsz                                = 407,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphDebugDotPrint                                      = 408,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamGetCaptureInfo_v2                                 = 409,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamGetCaptureInfo_v2_ptsz                            = 410,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamUpdateCaptureDependencies                         = 411,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamUpdateCaptureDependencies_ptsz                    = 412,
+    SANITIZER_CBID_RUNTIME_API_cudaUserObjectCreate                                        = 413,
+    SANITIZER_CBID_RUNTIME_API_cudaUserObjectRetain                                        = 414,
+    SANITIZER_CBID_RUNTIME_API_cudaUserObjectRelease                                       = 415,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphRetainUserObject                                   = 416,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphReleaseUserObject                                  = 417,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphInstantiateWithFlags                               = 418,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphAddMemAllocNode                                    = 419,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphMemAllocNodeGetParams                              = 420,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphAddMemFreeNode                                     = 421,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphMemFreeNodeGetParams                               = 422,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceGraphMemTrim                                      = 423,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceGetGraphMemAttribute                              = 424,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceSetGraphMemAttribute                              = 425,
+    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 426,
     SANITIZER_CBID_RUNTIME_API_FORCE_INT                                                   = 0x7fffffff
 } SANITIZER_CBID_RUNTIME_API;
 
