@@ -1,5 +1,5 @@
 /* Data structures associated with breakpoints in GDB.
-   Copyright (C) 1992-2019 Free Software Foundation, Inc.
+   Copyright (C) 1992-2020 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,41 +16,29 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-/*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2020 NVIDIA Corporation
- * Modified from the original GDB file referenced above by the CUDA-GDB 
- * team at NVIDIA <cudatools@nvidia.com>.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
- * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
- */
+/* NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2021 NVIDIA Corporation
+   Modified from the original GDB file referenced above by the CUDA-GDB
+   team at NVIDIA <cudatools@nvidia.com>. */
 
 #if !defined (BREAKPOINT_H)
 #define BREAKPOINT_H 1
 
 #include "frame.h"
 #include "value.h"
-#include "common/vec.h"
 #include "ax.h"
 #include "command.h"
-#include "common/break-common.h"
+#include "gdbsupport/break-common.h"
 #include "probe.h"
 #include "location.h"
 #include <vector>
-#include "common/array-view.h"
+#include "gdbsupport/array-view.h"
+#include "gdbsupport/function-view.h"
 #include "cli/cli-script.h"
-#include "cuda-tdep.h"
-
+#ifdef NVIDIA_CUDA_GDB
+#include "cuda/cuda-tdep.h"
 struct value;
+#endif
+
 struct block;
 struct gdbpy_breakpoint_object;
 struct gdbscm_breakpoint_object;
@@ -62,6 +50,16 @@ struct linespec_result;
 struct linespec_sals;
 struct inferior;
 
+/* Enum for exception-handling support in 'catch throw', 'catch rethrow',
+   'catch catch' and the MI equivalent.  */
+
+enum exception_event_kind
+{
+  EX_EVENT_THROW,
+  EX_EVENT_RETHROW,
+  EX_EVENT_CATCH
+};
+
 /* Why are we removing the breakpoint from the target?  */
 
 enum remove_bp_reason
@@ -162,28 +160,25 @@ enum bptype
 
     bp_thread_event,
 
+#ifdef NVIDIA_CUDA_GDB
     /* CUDA - auto breakpoints */
     /* With 'set cuda break_on_launch on', internal breakpoints are set
        on the entry address of all the CUDA kernels. Those breakpoints
        are marked bp_cuda_auto.  */
     bp_cuda_auto,
-
     /* CUDA - autostep */
     /* This is for single-step points (autostep). */
     bp_cuda_autostep,
-
     /* CUDA - breakpoint for error reporting */
     /* This is an internal breakpoint that is hit when a driver API
      * returns an error. */
     bp_cuda_driver_api_error,
-
     /* This is an internal breakpoint that is hit when there's a driver
      * internal error. */
     bp_cuda_driver_internal_error,
-
     /* CUDA - UVM */
     bp_cuda_uvm,
-
+#endif
     /* On the same principal, an overlay manager can arrange to call a
        magic location in the inferior whenever there is an interesting
        change in overlay status.  GDB can update its overlay tables
@@ -320,9 +315,10 @@ struct bp_target_info
   /* Flag that is true if the breakpoint should be left in place even
      when GDB is not connected.  */
   int persist;
-
+#ifdef NVIDIA_CUDA_GDB
   /* Owner of this target info */
   struct bp_location *owner;
+#endif
 };
 
 /* GDB maintains two types of information about each breakpoint (or
@@ -351,7 +347,12 @@ class bp_location
 public:
   bp_location () = default;
 
-  bp_location (breakpoint *owner);
+  /* Construct a bp_location with the type inferred from OWNER's
+     type.  */
+  explicit bp_location (breakpoint *owner);
+
+  /* Construct a bp_location with type TYPE.  */
+  bp_location (breakpoint *owner, bp_loc_type type);
 
   virtual ~bp_location ();
 
@@ -405,7 +406,7 @@ public:
   agent_expr_up cmd_bytecode;
 
   /* Signals that breakpoint conditions and/or commands need to be
-     re-synched with the target.  This has no use other than
+     re-synced with the target.  This has no use other than
      target-side breakpoints.  */
   bool needs_update = false;
 
@@ -416,18 +417,18 @@ public:
 
   /* Is this particular location enabled.  */
   bool enabled = false;
-
-  /* Nonzero if this breakpoint is now inserted.  */
+  
+  /* True if this breakpoint is now inserted.  */
   bool inserted = false;
 
-  /* Nonzero if this is a permanent breakpoint.  There is a breakpoint
+  /* True if this is a permanent breakpoint.  There is a breakpoint
      instruction hard-wired into the target's code.  Don't try to
      write another breakpoint instruction on top of it, or restore its
      value.  Step over it using the architecture's
      gdbarch_skip_permanent_breakpoint method.  */
   bool permanent = false;
 
-  /* Nonzero if this is not the first breakpoint in the list
+  /* True if this is not the first breakpoint in the list
      for the given address.  location of tracepoint can _never_
      be duplicated with other locations of tracepoints and other
      kinds of breakpoints, because two locations at the same
@@ -491,10 +492,6 @@ public:
 
   char *function_name = NULL;
 
-  // NS: Don't really need it, do we?
-  /* Source file name of this address.  */
-  // char *source_file;
-
   /* Details of the placed breakpoint, when inserted.  */
   bp_target_info target_info {};
 
@@ -536,16 +533,16 @@ public:
 
   /* The objfile the symbol or minimal symbol were found in.  */
   const struct objfile *objfile = NULL;
-
+#ifdef NVIDIA_CUDA_GDB
   /* CUDA - breakpoints */
   elf_image_t    cuda_elf_image = NULL;          /* ELF image of the address */
-
   /* CUDA - device breakpoints */
   /* Once the location has been resolved to a CUDA address, remember that.
      Used to avoid disabling the breakpoint when the ELF image
      of the device kernel/function is not available upon re-setting the
      breakpoint. */
   bool cuda_breakpoint = false;
+#endif
 };
 
 /* The possible return values for print_bpstat, print_it_normal,
@@ -653,7 +650,7 @@ struct breakpoint_ops
      `create_sals_from_location_default'.
 
      This function is called inside `create_breakpoint'.  */
-  void (*create_sals_from_location) (const struct event_location *location,
+  void (*create_sals_from_location) (struct event_location *location,
 				     struct linespec_result *canonical,
 				     enum bptype type_wanted);
 
@@ -680,7 +677,7 @@ struct breakpoint_ops
      This function is called inside `location_to_sals'.  */
   std::vector<symtab_and_line> (*decode_location)
     (struct breakpoint *b,
-     const struct event_location *location,
+     struct event_location *location,
      struct program_space *search_pspace);
 
   /* Return true if this breakpoint explains a signal.  See
@@ -701,13 +698,14 @@ struct breakpoint_ops
    specific extra command necessary for B's recreation.  */
 extern void print_recreate_thread (struct breakpoint *b, struct ui_file *fp);
 
+#ifdef NVIDIA_CUDA_GDB
 /* CUDA - autostep */
 enum cuda_autostep_length_type_t
 {
   cuda_autostep_insts,
   cuda_autostep_lines
 };
-
+#endif
 enum watchpoint_triggered
 {
   /* This watchpoint definitely did not trigger.  */
@@ -723,10 +721,10 @@ enum watchpoint_triggered
 
 /* Some targets (e.g., embedded PowerPC) need two debug registers to set
    a watchpoint over a memory region.  If this flag is true, GDB will use
-   only one register per watchpoint, thus assuming that all acesses that
+   only one register per watchpoint, thus assuming that all accesses that
    modify a memory location happen at its starting address. */
 
-extern int target_exact_watchpoints;
+extern bool target_exact_watchpoints;
 
 /* Note that the ->silent field is not currently used by any commands
    (though the code is in there if it was to be, and set_raw_breakpoint
@@ -785,9 +783,8 @@ struct breakpoint
   event_location_up location;
 
   /* The filter that should be passed to decode_line_full when
-     re-setting this breakpoint.  This may be NULL, but otherwise is
-     allocated with xmalloc.  */
-  char *filter = NULL;
+     re-setting this breakpoint.  This may be NULL.  */
+  gdb::unique_xmalloc_ptr<char> filter;
 
   /* For a ranged breakpoint, the location we used to find the end of
      the range.  */
@@ -821,27 +818,25 @@ struct breakpoint
      care.  */
   int task = 0;
 
+#ifdef NVIDIA_CUDA_GDB
   /* CUDA - breakpoints */
   /* Nonzero if the break command is set with an explicit device address */
   /* Such breakpoint will be disabled on the next run */
   char cuda_explicit_device_address = 0;
-
   /* CUDA - breakpoints */
   /* Indicates that a breakpoint has or had a device location and should
      not be disabled when all locations are removed (eg. when ELF image
      gets unloaded).  */
   bool cuda_breakpoint = false;
-
   /* CUDA - autostep */
   /* The length of the autostep */
   unsigned int cuda_autostep_length = 0;
-
   /* The type of length for the autostep, such as lines or instructions */
   enum cuda_autostep_length_type_t cuda_autostep_length_type = cuda_autostep_insts;
-
+#endif
   /* Count of the number of times this breakpoint was taken, dumped
-     with the info, but not used for anything else.  Useful for
-     seeing how many times you hit a break prior to the program
+     with the info, but not used for anything else.  Useful for seeing
+     how many times you hit a break prior to the program aborting, so
      you can back up to just before the abort.  */
   int hit_count = 0;
 
@@ -887,9 +882,10 @@ struct watchpoint : public breakpoint
      we do not know the value yet or the value was not readable.  VAL
      is never lazy.  */
   value_ref_ptr val;
-  /* Nonzero if VAL is valid.  If VAL_VALID is set but VAL is NULL,
+
+  /* True if VAL is valid.  If VAL_VALID is set but VAL is NULL,
      then an error occurred reading the value.  */
-  int val_valid;
+  bool val_valid;
 
   /* When watching the location of a bitfield, contains the offset and size of
      the bitfield.  Otherwise contains 0.  */
@@ -935,11 +931,17 @@ extern struct breakpoint *breakpoint_find_if
 /* Return true if BPT is either a software breakpoint or a hardware
    breakpoint.  */
 
-extern int is_breakpoint (const struct breakpoint *bpt);
+extern bool is_breakpoint (const struct breakpoint *bpt);
 
-/* Returns true if BPT is really a watchpoint.  */
+/* Return true if BPT is of any watchpoint kind, hardware or
+   software.  */
 
-extern int is_watchpoint (const struct breakpoint *bpt);
+extern bool is_watchpoint (const struct breakpoint *bpt);
+
+/* Return true if BPT is a C++ exception catchpoint (catch
+   catch/throw/rethrow).  */
+
+extern bool is_exception_catchpoint (breakpoint *bp);
 
 /* An instance of this type is used to represent all kinds of
    tracepoints.  */
@@ -1112,7 +1114,7 @@ struct bpstat_what
     /* Used for BPSTAT_WHAT_SET_LONGJMP_RESUME and
        BPSTAT_WHAT_CLEAR_LONGJMP_RESUME.  True if we are handling a
        longjmp, false if we are handling an exception.  */
-    int is_longjmp;
+    bool is_longjmp;
   };
 
 /* Tell what to do about this bpstat.  */
@@ -1125,18 +1127,18 @@ extern void bpstat_run_callbacks (bpstat bs_head);
 /* Find the bpstat associated with a breakpoint.  NULL otherwise.  */
 bpstat bpstat_find_breakpoint (bpstat, struct breakpoint *);
 
-/* Nonzero if a signal that we got in target_wait() was due to
+/* True if a signal that we got in target_wait() was due to
    circumstances explained by the bpstat; the signal is therefore not
    random.  */
-extern int bpstat_explains_signal (bpstat, enum gdb_signal);
+extern bool bpstat_explains_signal (bpstat, enum gdb_signal);
 
-/* Nonzero is this bpstat causes a stop.  */
-extern int bpstat_causes_stop (bpstat);
+/* True if this bpstat causes a stop.  */
+extern bool bpstat_causes_stop (bpstat);
 
-/* Nonzero if we should step constantly (e.g. watchpoints on machines
+/* True if we should step constantly (e.g. watchpoints on machines
    without hardware support).  This isn't related to a specific bpstat,
    just to things like whether watchpoints are set.  */
-extern int bpstat_should_step (void);
+extern bool bpstat_should_step ();
 
 /* Print a message indicating what happened.  Returns nonzero to
    say that only the source line should be printed after this (zero
@@ -1257,11 +1259,6 @@ enum breakpoint_here
 
 /* Prototypes for breakpoint-related functions.  */
 
-/* Return 1 if there's a program/permanent breakpoint planted in
-   memory at ADDRESS, return 0 otherwise.  */
-
-extern int program_breakpoint_here_p (struct gdbarch *gdbarch, CORE_ADDR address);
-
 extern enum breakpoint_here breakpoint_here_p (const address_space *,
 					       CORE_ADDR);
 
@@ -1362,9 +1359,6 @@ const char *bpdisp_text (enum bpdisp disp);
 
 extern void break_command (const char *, int);
 
-extern void hbreak_command_wrapper (const char *, int);
-extern void thbreak_command_wrapper (const char *, int);
-extern void rbreak_command_wrapper (const char *, int);
 extern void watch_command_wrapper (const char *, int, int);
 extern void awatch_command_wrapper (const char *, int, int);
 extern void rwatch_command_wrapper (const char *, int, int);
@@ -1418,6 +1412,15 @@ extern void init_catchpoint (struct breakpoint *b,
 extern void install_breakpoint (int internal, std::unique_ptr<breakpoint> &&b,
 				int update_gll);
 
+/* Returns the breakpoint ops appropriate for use with with LOCATION and
+   according to IS_TRACEPOINT.  Use this to ensure, for example, that you pass
+   the correct ops to create_breakpoint for probe locations.  If LOCATION is
+   NULL, returns bkpt_breakpoint_ops (or tracepoint_breakpoint_ops, if
+   IS_TRACEPOINT is true).  */
+
+extern const struct breakpoint_ops *breakpoint_ops_for_event_location
+  (const struct event_location *location, bool is_tracepoint);
+
 /* Flags that can be passed down to create_breakpoint, etc., to affect
    breakpoint creation in several ways.  */
 
@@ -1448,7 +1451,7 @@ enum breakpoint_create_flags
    Returns true if any breakpoint was created; false otherwise.  */
 
 extern int create_breakpoint (struct gdbarch *gdbarch,
-			      const struct event_location *location,
+			      struct event_location *location,
 			      const char *cond_string, int thread,
 			      const char *extra_string,
 			      int parse_extra,
@@ -1464,21 +1467,16 @@ extern void insert_breakpoints (void);
 
 extern int remove_breakpoints (void);
 
+#ifdef NVIDIA_CUDA_GDB
 /* CUDA - breakpoints */
 /* These functions behave the same as the ones above, except that
    the host watchpoints are skipped. */
 extern void cuda_insert_breakpoints (void);
-
 extern int cuda_remove_breakpoints (void);
+#endif
+/* Remove breakpoints of inferior INF.  */
 
-/* CUDA - breakpoints */
-/* These functions behave the same as the ones above, except that
-   the host watchpoints are skipped. */
-extern void cuda_insert_breakpoints (void);
-
-extern int cuda_remove_breakpoints (void);
-
-extern int remove_breakpoints_inf (inferior *inf);
+extern void remove_breakpoints_inf (inferior *inf);
 
 /* This function can be used to update the breakpoint package's state
    after an exec() system call has been executed.
@@ -1620,15 +1618,13 @@ extern struct breakpoint *create_and_insert_solib_event_breakpoint
 extern struct breakpoint *create_thread_event_breakpoint (struct gdbarch *,
 							  CORE_ADDR);
 
+#ifdef NVIDIA_CUDA_GDB
 /* CUDA - breakpoint for error reporting */
-extern void create_cuda_driver_api_error_breakpoint (struct gdbarch *,
-                                                     CORE_ADDR);
-extern void create_cuda_driver_internal_error_breakpoint (struct gdbarch *,
-                                                          CORE_ADDR);
+extern void create_cuda_driver_api_error_breakpoint (void);
+extern void create_cuda_driver_internal_error_breakpoint (void);
 extern void create_cuda_uvm_breakpoint (struct gdbarch *, CORE_ADDR);
-
 extern void update_cuda_driver_api_error_breakpoint (void);
-
+#endif
 extern void remove_jit_event_breakpoints (void);
 
 extern void remove_solib_event_breakpoints (void);
@@ -1639,8 +1635,9 @@ extern void remove_solib_event_breakpoints_at_next_stop (void);
 
 extern void disable_breakpoints_in_shlibs (void);
 
-/* This function returns TRUE if ep is a catchpoint.  */
-extern int is_catchpoint (struct breakpoint *);
+/* This function returns true if B is a catchpoint.  */
+
+extern bool is_catchpoint (struct breakpoint *b);
 
 /* Shared helper function (MI and CLI) for creating and installing
    a shared object event catchpoint.  */
@@ -1719,7 +1716,9 @@ extern struct tracepoint *
 /* Return a vector of all tracepoints currently defined.  */
 extern std::vector<breakpoint *> all_tracepoints (void);
 
-extern int is_tracepoint (const struct breakpoint *b);
+/* Return true if B is of tracepoint kind.  */
+
+extern bool is_tracepoint (const struct breakpoint *b);
 
 /* Return a vector of all static tracepoints defined at ADDR.  */
 extern std::vector<breakpoint *> static_tracepoints_here (CORE_ADDR addr);
@@ -1745,8 +1744,8 @@ public:
    returned.  This can be useful for implementing a search for a
    breakpoint with arbitrary attributes, or for applying an operation
    to every breakpoint.  */
-extern struct breakpoint *iterate_over_breakpoints (int (*) (struct breakpoint *,
-							     void *), void *);
+extern struct breakpoint *iterate_over_breakpoints
+  (gdb::function_view<bool (breakpoint *)>);
 
 /* Nonzero if the specified PC cannot be a location where functions
    have been inlined.  */
@@ -1774,21 +1773,34 @@ extern void maybe_print_thread_hit_breakpoint (struct ui_out *uiout);
 /* Print the specified breakpoint.  */
 extern void print_breakpoint (breakpoint *bp);
 
+#ifdef NVIDIA_CUDA_GDB
 /* CUDA - autostep */
 struct breakpoint *cuda_find_autostep_by_addr (CORE_ADDR address);
-
 /* CUDA - breakpoint */
 bool cuda_eval_thread_at_breakpoint (uint64_t pc, cuda_coords_t *c, int b_number);
-
 /* CUDA - auto breakpoints */
 void cuda_auto_breakpoints_add_locations (void);
 void cuda_auto_breakpoints_remove_locations (elf_image_t elf_image);
 void cuda_auto_breakpoints_update_breakpoints (void);
 void cuda_auto_breakpoints_cleanup_breakpoints (void);
-
-extern void delete_command (const char *arg, int from_tty);
-
+extern void cuda_delete_command (const char *arg, int from_tty);
+#endif
 /* Command element for the 'commands' command.  */
 extern cmd_list_element *commands_cmd_element;
 
+/* Whether to use the fixed output when printing information about a
+   multi-location breakpoint (see PR 9659).  */
+
+extern bool fix_multi_location_breakpoint_output_globally;
+
+/* Deal with "catch catch", "catch throw", and "catch rethrow" commands and
+   the MI equivalents.  Sets up to catch events of type EX_EVENT.  When
+   TEMPFLAG is true only the next matching event is caught after which the
+   catch-point is deleted.  If REGEX is not NULL then only exceptions whose
+   type name matches REGEX will trigger the event.  */
+
+extern void catch_exception_event (enum exception_event_kind ex_event,
+				   const char *regex, bool tempflag,
+				   int from_tty);
+
 #endif /* !defined (BREAKPOINT_H) */
