@@ -1,5 +1,5 @@
 /* Darwin support for GDB, the GNU debugger.
-   Copyright (C) 1997-2019 Free Software Foundation, Inc.
+   Copyright (C) 1997-2020 Free Software Foundation, Inc.
 
    Contributed by Apple Computer, Inc.
 
@@ -43,13 +43,15 @@
 #include "amd64-darwin-tdep.h"
 #endif
 
-struct i386_darwin_nat_target /* final */ : public x86_nat_target<darwin_nat_target>
+struct i386_darwin_nat_target final : public x86_nat_target<darwin_nat_target>
 {
   /* Add our register access methods.  */
   void fetch_registers (struct regcache *, int) override;
   void store_registers (struct regcache *, int) override;
 };
 
+static struct i386_darwin_nat_target darwin_target;
+
 /* Read register values from the inferior process.
    If REGNO is -1, do this for all registers.
    Otherwise, REGNO specifies which register (so we can save time).  */
@@ -78,7 +80,7 @@ i386_darwin_nat_target::fetch_registers (struct regcache *regcache, int regno)
 	  ret = thread_get_state
             (current_thread, x86_THREAD_STATE, (thread_state_t) & gp_regs,
              &gp_count);
-	  if (ret != KERN_SUCCESS && ret != KERN_INVALID_ARGUMENT)
+	  if (ret != KERN_SUCCESS)
 	    {
 	      printf_unfiltered (_("Error calling thread_get_state for "
 				   "GP registers for thread 0x%lx\n"),
@@ -103,7 +105,7 @@ i386_darwin_nat_target::fetch_registers (struct regcache *regcache, int regno)
 	  ret = thread_get_state
             (current_thread, x86_FLOAT_STATE, (thread_state_t) & fp_regs,
              &fp_count);
-	  if (ret != KERN_SUCCESS && ret != KERN_INVALID_ARGUMENT)
+	  if (ret != KERN_SUCCESS)
 	    {
 	      printf_unfiltered (_("Error calling thread_get_state for "
 				   "float registers for thread 0x%lx\n"),
@@ -127,7 +129,7 @@ i386_darwin_nat_target::fetch_registers (struct regcache *regcache, int regno)
 	  ret = thread_get_state
             (current_thread, x86_THREAD_STATE32, (thread_state_t) &gp_regs,
              &gp_count);
-	  if (ret != KERN_SUCCESS && ret != KERN_INVALID_ARGUMENT)
+	  if (ret != KERN_SUCCESS)
 	    {
 	      printf_unfiltered (_("Error calling thread_get_state for "
 				   "GP registers for thread 0x%lx\n"),
@@ -151,7 +153,7 @@ i386_darwin_nat_target::fetch_registers (struct regcache *regcache, int regno)
 	  ret = thread_get_state
             (current_thread, x86_FLOAT_STATE32, (thread_state_t) &fp_regs,
              &fp_count);
-	  if (ret != KERN_SUCCESS && ret != KERN_INVALID_ARGUMENT)
+	  if (ret != KERN_SUCCESS)
 	    {
 	      printf_unfiltered (_("Error calling thread_get_state for "
 				   "float registers for thread 0x%lx\n"),
@@ -285,46 +287,18 @@ i386_darwin_dr_set (int regnum, CORE_ADDR value)
 {
   thread_t current_thread;
   x86_debug_state_t dr_regs;
-  x86_thread_state_t regs;
-  unsigned int count = x86_THREAD_STATE_COUNT;
   kern_return_t ret;
+  unsigned int dr_count;
 
   gdb_assert (regnum >= 0 && regnum <= DR_CONTROL);
 
   current_thread = inferior_ptid.tid ();
 
-  ret = thread_get_state (current_thread, x86_THREAD_STATE,
-                          (thread_state_t) &regs, &count);
-
-  if (ret != KERN_SUCCESS)
-    {
-      printf_unfiltered (_("i386_darwin_dr_set: error %x, thread=%x\n"),
-                        ret, current_thread);
-      return;
-    }
-
-  if (regs.tsh.flavor != x86_THREAD_STATE32 && 
-      regs.tsh.flavor != x86_THREAD_STATE64 )
-    {
-      printf_unfiltered (
-          _("i386_darwin_dr_set: invalid thread 0x%x flavor %d\n"),
-          current_thread, regs.tsh.flavor);
-      return;
-    }
-
-  if (regs.tsh.flavor == x86_THREAD_STATE32)
-    {
-      dr_regs.dsh.flavor = x86_DEBUG_STATE32;
-      dr_regs.dsh.count = x86_DEBUG_STATE32_COUNT;
-    }
-  else
-    {
-      dr_regs.dsh.flavor = x86_DEBUG_STATE64;
-      dr_regs.dsh.count = x86_DEBUG_STATE64_COUNT;
-    }
-  count = x86_DEBUG_STATE_COUNT;
+  dr_regs.dsh.flavor = x86_DEBUG_STATE;
+  dr_regs.dsh.count = x86_DEBUG_STATE_COUNT;
+  dr_count = x86_DEBUG_STATE_COUNT;
   ret = thread_get_state (current_thread, x86_DEBUG_STATE,
-                          (thread_state_t) &dr_regs, &count);
+                          (thread_state_t) &dr_regs, &dr_count);
 
   MACH_CHECK_ERROR (ret);
 
@@ -393,7 +367,7 @@ i386_darwin_dr_set (int regnum, CORE_ADDR value)
     }
 
   ret = thread_set_state (current_thread, dr_regs.dsh.flavor,
-                          (thread_state_t) &dr_regs.uds, count);
+                          (thread_state_t) &dr_regs.uds, dr_count);
 
   MACH_CHECK_ERROR (ret);
 }
@@ -403,47 +377,18 @@ i386_darwin_dr_get (int regnum)
 {
   thread_t current_thread;
   x86_debug_state_t dr_regs;
-  x86_thread_state_t regs;
-  unsigned int count = x86_THREAD_STATE_COUNT;
   kern_return_t ret;
+  unsigned int dr_count;
 
   gdb_assert (regnum >= 0 && regnum <= DR_CONTROL);
 
   current_thread = inferior_ptid.tid ();
 
-  ret = thread_get_state (current_thread, x86_THREAD_STATE,
-			   (thread_state_t) &regs, &count);
-  if (ret != KERN_SUCCESS)
-    {
-      printf_unfiltered (_("i386_darwin_dr_get: error %x, thread=%x\n"),
-			 ret, current_thread);
-      return -1;
-    }
-
-  if (regs.tsh.flavor != x86_THREAD_STATE32 &&
-      regs.tsh.flavor != x86_THREAD_STATE64 )
-    {
-      printf_unfiltered (
-          _("i386_darwin_dr_get: invalid thread 0x%x flavor %d\n"),
-          current_thread, regs.tsh.flavor);
-      return -1;
-    }
-
-  if (regs.tsh.flavor == x86_THREAD_STATE32)
-    {
-      dr_regs.dsh.flavor = x86_DEBUG_STATE32;
-      dr_regs.dsh.count = x86_DEBUG_STATE32_COUNT;
-    }
-  else
-    {
-      dr_regs.dsh.flavor = x86_DEBUG_STATE64;
-      dr_regs.dsh.count = x86_DEBUG_STATE64_COUNT;
-    }
-
-  count = x86_DEBUG_STATE_COUNT;
+  dr_regs.dsh.flavor = x86_DEBUG_STATE;
+  dr_regs.dsh.count = x86_DEBUG_STATE_COUNT;
+  dr_count = x86_DEBUG_STATE_COUNT;
   ret = thread_get_state (current_thread, x86_DEBUG_STATE,
-                          (thread_state_t) &dr_regs, &count);
-
+                          (thread_state_t) &dr_regs, &dr_count);
   MACH_CHECK_ERROR (ret);
 
   switch (dr_regs.dsh.flavor)
@@ -694,13 +639,9 @@ darwin_set_sstep (thread_t thread, int enable)
     }
 }
 
-#include "cuda-linux-nat-template.c"
-
-/* Was: static struct i386_darwin_nat_target darwin_target; */
-static cuda_nat_linux<i386_darwin_nat_target> darwin_target;
-
+void _initialize_i386_darwin_nat ();
 void
-_initialize_i386_darwin_nat (void)
+_initialize_i386_darwin_nat ()
 {
 #ifdef BFD64
   amd64_native_gregset64_reg_offset = amd64_darwin_thread_state_reg_offset;
