@@ -1,6 +1,6 @@
 /* Linux-specific functions to retrieve OS data.
 
-   Copyright (C) 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2009-2020 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,11 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "common/common-defs.h"
+/* NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2021 NVIDIA Corporation
+   Modified from the original GDB file referenced above by the CUDA-GDB
+   team at NVIDIA <cudatools@nvidia.com>. */
+
+#include "gdbsupport/common-defs.h"
 #include "linux-osdata.h"
 
 #include <sys/types.h>
@@ -32,11 +36,11 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "common/xml-utils.h"
-#include "common/buffer.h"
+#include "gdbsupport/xml-utils.h"
+#include "gdbsupport/buffer.h"
 #include <dirent.h>
 #include <sys/stat.h>
-#include "common/filestuff.h"
+#include "gdbsupport/filestuff.h"
 #include <algorithm>
 
 #define NAMELEN(dirent) strlen ((dirent)->d_name)
@@ -205,7 +209,10 @@ commandline_from_pid (PID_T pid)
 static void
 user_from_uid (char *user, int maxlen, uid_t uid)
 {
-  struct passwd *pwentry = getpwuid (uid);
+  struct passwd *pwentry;
+  char buf[1024];
+  struct passwd pwd;
+  getpwuid_r (uid, &pwd, buf, sizeof (buf), &pwentry);
 
   if (pwentry)
     {
@@ -265,8 +272,9 @@ get_cores_used_by_process (PID_T pid, int *cores, const int num_cores)
 	  core = linux_common_core_of_thread (ptid_t ((pid_t) pid,
 						      (pid_t) tid, 0));
 
+#ifdef NVIDIA_CUDA_GDB
 	  if (core >= 0)
-	    {
+	    { 
 	      if (core >= num_cores)
 		{
 	          printf ("Warning: Adjusting return value of linux_common_core_of_thread (pid=%lu, tid=%lu).\n"
@@ -274,6 +282,10 @@ get_cores_used_by_process (PID_T pid, int *cores, const int num_cores)
 	                  (unsigned long)pid, (unsigned long)tid, core, num_cores);
 	          core = num_cores-1;
 		}
+#else
+	  if (core >= 0 && core < num_cores)
+	    {
+#endif
 	      ++cores[core];
 	      ++task_count;
 	    }
@@ -295,11 +307,11 @@ linux_xfer_osdata_processes (struct buffer *buffer)
   dirp = opendir ("/proc");
   if (dirp)
     {
-#ifndef __arm__
-      const int num_cores = sysconf (_SC_NPROCESSORS_ONLN);
-#else
+#if defined(NVIDIA_CUDA_GDB) && defined(__arm__)
       /* On ARM, count both online and offline CPUs */
       const int num_cores = sysconf (_SC_NPROCESSORS_CONF);
+#else
+      const int num_cores = sysconf (_SC_NPROCESSORS_ONLN);
 #endif
       struct dirent *dp;
 
@@ -578,11 +590,12 @@ linux_xfer_osdata_cpus (struct buffer *buffer)
 	      char *key, *value;
 	      int i = 0;
 
-	      key = strtok (buf, ":");
+	      char *saveptr;
+	      key = strtok_r (buf, ":", &saveptr);
 	      if (key == NULL)
 		continue;
 
-	      value = strtok (NULL, ":");
+	      value = strtok_r (NULL, ":", &saveptr);
 	      if (value == NULL)
 		continue;
 
@@ -924,7 +937,11 @@ time_from_time_t (char *time, int maxlen, TIME_T seconds)
     {
       time_t t = (time_t) seconds;
 
-      strncpy (time, ctime (&t), maxlen);
+      /* Per the ctime_r manpage, this buffer needs to be at least 26
+         characters long.  */
+      char buf[30];
+      const char *time_str = ctime_r (&t, buf);
+      strncpy (time, time_str, maxlen);
       time[maxlen - 1] = '\0';
     }
 }
@@ -1317,7 +1334,7 @@ struct osdata_type {
   { "sockets", "Sockets", "Listing of all internet-domain sockets",
     linux_xfer_osdata_isockets, -1 },
   { "threads", "Threads", "Listing of all threads",
-    linux_xfer_osdata_threads, -1 },
+  linux_xfer_osdata_threads, -1 },
   { NULL, NULL, NULL }
 };
 
@@ -1343,6 +1360,7 @@ linux_xfer_osdata_info_os_types (struct buffer *buffer)
   buffer_grow_str0 (buffer, "</osdata>\n");
 }
 
+
 /*  Copies up to LEN bytes in READBUF from offset OFFSET in OSD->BUFFER.
     If OFFSET is zero, first calls OSD->TAKE_SNAPSHOT.  */
 
