@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2020 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2015-2021 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,13 @@
 #include "language.h"
 #include "python.h"
 #include "python-internal.h"
+#include "gdbsupport/scope-exit.h"
 #include <dlfcn.h>
+#ifdef IS_PY3K
+#include <regex>
+#include <string>
+#include <vector>
+#endif
 
 #if HAVE_LIBPYTHON2_4
 #define CONSTCHAR char
@@ -81,8 +87,6 @@ char * (**pgdbpyOS_ReadlineFunctionPointer) (FILE *, FILE *,
 			char *) = NULL;
 #endif
 
-PyThreadState **gdbpy_ThreadState_Current = NULL;
-
 /* Imported functions */
 int (*gdbpy_Arg_UnpackTuple) (PyObject *, const char *, Py_ssize_t, Py_ssize_t, ...) = NULL;
 PyObject * (*gdbpy_ErrFormat)(PyObject *, const char *, ...) = NULL;
@@ -93,9 +97,19 @@ PyObject * (*gdbpy_PyObject_CallMethod)(PyObject *o, const char *m, const char *
 PyObject * (*gdbpy_PyErr_NewException) (const char *name, PyObject *base, PyObject *dict) = NULL;
 PyObject * (*gdbpy_PyString_FromStringAndSize) (const char *name, Py_ssize_t size) = NULL;
 PyObject * (*gdbpy_PyBytes_FromStringAndSize) (const char *name, Py_ssize_t size) = NULL;
+#ifdef IS_PY3K
+//char * (*gdbpy_PyBytes_AsString) (PyObject *o) = NULL;
+PyObject * (*gdbpy_PyBytes_FromString) (const char *v) = NULL;
+#endif
 
 #ifdef IS_PY3K
-PyObject * (*gdbpy_PyMemoryView_FromObject) () = NULL;
+void (*gdbpy_SetProgramName) (wchar_t *) = NULL;
+#else
+void (*gdbpy_SetProgramName) (char *) = NULL;
+#endif
+
+#ifdef IS_PY3K
+PyObject * (*gdbpy_PyMemoryView_FromObject) (PyObject *) = NULL;
 int (*gdbpy_PySlice_GetIndicesEx) (PyObject *slice, Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step, Py_ssize_t *slicelength) = NULL;
 #else
 PyObject * (*gdbpy_PyBuffer_FromObject) (PyObject *base, Py_ssize_t offset, Py_ssize_t size) = NULL;
@@ -135,9 +149,19 @@ Py_ssize_t (*gdbpy_PyInt_AsSize_t) (PyObject *) = NULL;
 PyObject * (*gdbpy_PyInt_FromSize_t) (size_t size) = NULL;
 PyObject * (*gdbpy_PyInt_FromLong) (long val) = NULL;
 int (*gdbpy_PyObject_GetBuffer) (PyObject *o, Py_buffer *buf, int f) = NULL;
+Py_ssize_t (*gdbpy_PyObject_Length)(PyObject *o) = NULL;
+#ifdef IS_PY3K
+int (*gdbpy_PyBuffer_FillInfo) (Py_buffer *view, PyObject *exporter, void *buf, Py_ssize_t len, int readonly, int flags) = NULL;
+#endif
 void (*gdbpy_PyBuffer_Release) (Py_buffer *buf) = NULL;
+int (*gdbpy_PyErr_GivenExceptionMatches) (PyObject *, PyObject *) = NULL;
 
 static PyObject * (*gdb_PyBool_FromLong) (long) = NULL;
+#ifdef IS_PY3K
+static char * (*gdb_PyBytes_AsString) (PyObject *o) = NULL;
+static PyObject * (*gdb_PyBytes_FromString) (const char *v) = NULL;
+static int (*gdb_PyBuffer_FillInfo) (Py_buffer *view, PyObject *exporter, void *buf, Py_ssize_t len, int readonly, int flags) = NULL;
+#endif
 static PyObject * (*gdb_PyBuffer_FromReadWriteObject) (PyObject *base, Py_ssize_t offset, Py_ssize_t size) = NULL;
 static int (*gdb_PyCallable_Check) (PyObject *o) = NULL;
 static PyObject * (*gdb_PyDict_New) (void) = NULL;
@@ -165,7 +189,11 @@ static PyGILState_STATE (*gdb_PyGILState_Ensure) (void) = NULL;
 static void (*gdb_PyGILState_Release) (PyGILState_STATE) = NULL;
 static PyObject * (*gdb_PyImport_AddModule) (const char *name) = NULL;
 static PyObject * (*gdb_PyImport_ImportModule) (const char *name) = NULL;
+#ifdef IS_PY3K
+static int (*gdb_PyImport_AppendInittab) (const char *name, PyObject *(*initfunc)(void)) = NULL;
+#else
 static long (*gdb_PyInt_AsLong) (PyObject *) = NULL;
+#endif
 static PyObject * (*gdb_PyInt_FromLong) (long) = NULL;
 static long (*gdb_PyInt_GetMax) (void) = NULL;
 static PyObject * (*gdb_PyIter_Next) (PyObject *) = NULL;
@@ -176,11 +204,18 @@ static int (*gdb_PyList_Insert) (PyObject *, Py_ssize_t, PyObject *) = NULL;
 static PyObject * (*gdb_PyList_New) (Py_ssize_t size) = NULL;
 static Py_ssize_t (*gdb_PyList_Size) (PyObject *) = NULL;
 static PY_LONG_LONG (*gdb_PyLong_AsLongLong) (PyObject *) = NULL;
+#ifdef IS_PY3K
+static long (*gdb_PyLong_AsLong) (PyObject *) = NULL;
+#endif
 static unsigned PY_LONG_LONG (*gdb_PyLong_AsUnsignedLongLong) (PyObject *) = NULL;
 static PyObject * (*gdb_PyLong_FromLong) (long) = NULL;
 static PyObject * (*gdb_PyLong_FromLongLong) (PY_LONG_LONG) = NULL;
 static PyObject * (*gdb_PyLong_FromUnsignedLong) (unsigned long) = NULL;
 static PyObject * (*gdb_PyLong_FromUnsignedLongLong) (unsigned PY_LONG_LONG) = NULL;
+#ifdef IS_PY3K
+static PyObject * (*gdb_PyMemoryView_FromObject) (PyObject *) = NULL;
+static void * (*gdb_PyMem_RawMalloc) (size_t) = NULL;
+#endif
 static void * (*gdb_PyMem_Malloc) (size_t) = NULL;
 static int (*gdb_PyModule_AddIntConstant) (PyObject *, const char *, long) = NULL;
 static int (*gdb_PyModule_AddObject) (PyObject *, const char *, PyObject *) = NULL;
@@ -231,7 +266,12 @@ static int (*gdb_Py_FlushLine) (void) = NULL;
 static void (*gdb_Py_Initialize) (void) = NULL;
 static PyObject * (*gdb_Py_InitModule4)(const char *, PyMethodDef *, const char *, PyObject *, int) = NULL;
 static PyObject * (*gdb_Py_InitModule4_64)(const char *, PyMethodDef *, const char *, PyObject *, int) = NULL;
+#ifdef IS_PY3K
+static PyObject * (*gdb_PyModule_Create2)(PyModuleDef *, int) = NULL;
+static void (*gdb_Py_SetProgramName) (wchar_t *) = NULL;
+#else
 static void (*gdb_Py_SetProgramName) (char *) = NULL;
+#endif
 static PyObject * (*gdb__PyObject_New) (PyTypeObject *) = NULL;
 static PyCodeObject * (*gdb_PyCode_New) (int, int, int, int,
            PyObject *, PyObject *, PyObject *, PyObject *,
@@ -242,8 +282,30 @@ static PyObject* (*gdb_PyUnicode_Decode)(const char *s, Py_ssize_t size, const c
 static PyObject* (*gdb_PyUnicode_AsEncodedString)(register PyObject *unicode, const char *encoding, const char *errors) = NULL;
 static PyObject* (*gdb_PyUnicode_FromEncodedObject)(register PyObject *obj, const char *encoding, const char *errors) = NULL;
 static int *gdb_Py_DontWriteBytecodeFlag = NULL;
+#ifdef IS_PY3K
+static PyObject* (*gdb_PyUnicode_FromString)(const char *string) = NULL;
+static int (*gdb_PyUnicode_CompareWithASCIIString)(PyObject *uni, const char *string) = NULL;
+#endif
+static void (*gdb_PyBuffer_Release) (Py_buffer *buf) = NULL;
 
+#ifdef IS_PY3K
+/* base libpython name */
+static const std::string libpython3_base = "libpython";
 
+/* Possible PEP-3149 flag names */
+static const std::vector<std::string> libpython3_flags = {
+  "", "d", "m", "u",
+  "dm", "du", "mu",
+  "dmu"
+};
+
+/* Possible soname endings */
+static const std::vector<std::string> libpython3_end = {
+  ".so", ".so.1", ".so.1.0"
+};
+
+static const char py_vers_error[] = "Unable to determine python3 interpreter version. Python integration disabled.";
+#endif
 
 bool
 is_python_available (void) {
@@ -266,8 +328,78 @@ is_python_available (void) {
   if (python_initialized)
     return libpython_handle != NULL;
 
+#if IS_PY3K
+  /* Determine the python version by calling out to python3 */
+  FILE *py_vers_file = popen ("python3 -V 2> /dev/null", "r");
+  if (!py_vers_file)
+    {
+      fprintf (stderr, "%s\n", py_vers_error);
+      python_initialized = true;
+      return false;
+    }
+  /* Cleanup for me please. */
+  auto cleanup = make_scope_exit ([&] ()
+    {
+      pclose (py_vers_file);
+    });
+
+  /* Read the version string */
+  char py_vers_buf[BUFSIZ] = {};
+  if (!fgets (py_vers_buf, BUFSIZ, py_vers_file))
+    {
+      fprintf (stderr, "%s\n", py_vers_error);
+      python_initialized = true;
+      return false;
+    }
+
+  /* Capture the major.minor version we are interested in and get the key string. */
+  const std::string py_vers_str{py_vers_buf};
+  const std::regex py_vers_regex{"Python ([0-9]+\\.[0-9]+)"};
+  std::smatch py_vers_match;
+  std::string py_vers;
+  if (std::regex_search (py_vers_str.begin(), py_vers_str.end(), py_vers_match, py_vers_regex) && 
+      py_vers_match.size() == 2)
+    {
+      /* First result is the entire string, we want the second one. */
+      auto sub_match = py_vers_match[1];
+      py_vers = sub_match.str();
+    }
+  else
+    {
+      /* Failed to parse the version. */
+      fprintf (stderr, "%s\n", py_vers_error);
+      python_initialized = true;
+      return false;
+    }
+
+  /* Try to dlopen the appropraite libraries based on the python interpreter version.
+   * We will construct the possible libpython name from parts. */
+  const auto libpython3_libname = libpython3_base + py_vers;
+  /* Start with the possible flags. We also include the empty string here. */
+  for ( const auto& it : libpython3_flags )
+    {
+      const auto fname = libpython3_libname + it;
+      /* Add each possible ending - This is probably overkill. */
+      for ( const auto& it : libpython3_end )
+	{
+	  const auto full_name = fname + it;
+	  libpython_handle = dlopen (full_name.c_str(), RTLD_NOW | RTLD_GLOBAL);
+	  /* Short circuit if we dlopened it. */
+	  if (libpython_handle)
+	    break;
+	}
+      /* Final short circuit if we found the library. */
+      if (libpython_handle)
+	break;
+    }
+  /* If we didn't find a library - try to dlopen libpython3.so as a final check. This exists on some distros. */
+  if (libpython_handle == NULL)
+    libpython_handle = dlopen ("libpython3.so", RTLD_NOW | RTLD_GLOBAL);
+#else
+  /* Python 2 tries to dlopen everything - not a good idea but python2 support has been removed from cuda-gdb. */
   for ( i = 0; libpython_names[i] && !libpython_handle ; ++i)
     libpython_handle = dlopen ( libpython_names[i], RTLD_NOW | RTLD_GLOBAL);
+#endif
 
   python_initialized = true;
   if (!libpython_handle)
@@ -285,19 +417,26 @@ is_python_available (void) {
   /* Resolve types and exceptions */
   RESOLVE_AND_CHECK(gdbpy_None, PyObject *, "_Py_NoneStruct");
   RESOLVE_AND_CHECK(gdbpy_True, PyObject *, "_Py_TrueStruct");
+#ifdef IS_PY3K
+  RESOLVE_AND_CHECK(gdbpy_False, PyObject *, "_Py_FalseStruct");
+#else
   RESOLVE_AND_CHECK(gdbpy_Zero, PyObject *, "_Py_ZeroStruct");
+#endif
 
   RESOLVE_AND_CHECK(gdbpy_FloatType, PyTypeObject *, "PyFloat_Type");
   RESOLVE_AND_CHECK(gdbpy_BoolType, PyTypeObject *, "PyBool_Type");
+#ifndef IS_PY3K
   RESOLVE_AND_CHECK(gdbpy_IntType, PyTypeObject *, "PyInt_Type");
+#endif
   RESOLVE_AND_CHECK(gdbpy_LongType, PyTypeObject *, "PyLong_Type");
   RESOLVE_AND_CHECK(gdbpy_SliceType, PyTypeObject *, "PySlice_Type");
+#ifndef IS_PY3K
   RESOLVE_AND_CHECK(gdbpy_StringType, PyTypeObject *, "PyString_Type");
+#endif
   RESOLVE_AND_CHECK(gdbpy_ListType, PyTypeObject *, "PyList_Type");
   RESOLVE_AND_CHECK(gdbpy_TupleType, PyTypeObject *, "PyTuple_Type");
   RESOLVE_AND_CHECK(gdbpy_UnicodeType, PyTypeObject *, "PyUnicode_Type");
   RESOLVE_AND_CHECK(gdbpy_NotImplemented, PyObject *, "_Py_NotImplementedStruct");
-  RESOLVE_AND_CHECK(gdbpy_ThreadState_Current, PyThreadState **, "_PyThreadState_Current");
 
   RESOLVE_AND_CHECK(pgdbpyExc_AttributeError, PyObject **, "PyExc_AttributeError");
   RESOLVE_AND_CHECK(pgdbpyExc_IndexError, PyObject **, "PyExc_IndexError");
@@ -332,7 +471,9 @@ is_python_available (void) {
   RESOLVE_AND_CHECK(gdbpy_PyArg_ParseTuple, int (*) (PyObject *obj, const char *, ...), STRINGIFY(PyArg_ParseTuple));
   RESOLVE_AND_CHECK(gdbpy_PyArg_ParseTupleAndKeywords, int (*) (PyObject *obj, PyObject *, const char *, char **, ...), STRINGIFY(PyArg_ParseTupleAndKeywords));
   RESOLVE_AND_CHECK(gdbpy_PyArg_VaParseTupleAndKeywords, int (*) (PyObject *obj, PyObject *, const char *, char **, ...), STRINGIFY(PyArg_VaParseTupleAndKeywords));
+#ifndef IS_PY3K
   RESOLVE_AND_CHECK(gdbpy_StringFromFormat, PyObject * (*)  (const char *, ...), "PyString_FromFormat");
+#endif
 
 #define RESOLVE(varname,type,symname)				\
   varname = (type) dlsym (libpython_handle, symname);
@@ -347,20 +488,30 @@ is_python_available (void) {
   RESOLVE(gdbpy_PyInt_FromSize_t, PyObject * (*) (size_t), "PyInt_FromSize_t");
   RESOLVE(gdbpy_PyInt_FromLong, PyObject * (*) (long), "PyInt_FromLong");
   RESOLVE(gdbpy_PyObject_GetBuffer, int (*) (PyObject *, Py_buffer *, int), "PyObject_GetBuffer");
+  RESOLVE(gdbpy_PyObject_Length, Py_ssize_t (*) (PyObject *o), "PyObject_Length");
+#ifdef IS_PY3K
+  RESOLVE(gdbpy_PyBuffer_FillInfo, int (*) (Py_buffer *view, PyObject *exporter, void *buf, Py_ssize_t len, int readonly, int flags), "PyBuffer_FillInfo");
+#endif
   RESOLVE(gdbpy_PyBuffer_Release, void (*) (Py_buffer *buf), "PyBuffer_Release");
-
-  RESOLVE(gdbpy_Long_AsLong, PY_LONG_LONG (*) (PyObject *), "PyLong_AsLong");
+  RESOLVE(gdb_PyBuffer_Release, void (*) (Py_buffer *buf), "PyBuffer_Release");
+  RESOLVE(gdbpy_PyErr_GivenExceptionMatches, int (*) (PyObject *, PyObject *), "PyErr_GivenExceptionMatches");
+  RESOLVE(gdbpy_Long_AsLong, PY_LONG_LONG (*) (PyObject *), "PyLong_AsLongLong");
   RESOLVE(gdbpy_PyObject__IsInstance, int (*) (PyObject *object, PyObject *typeorclass), "PyObject_IsInstance");
   RESOLVE(gdbpy_PySequence_Concat, PyObject * (*) (PyObject *o1, PyObject *o2), "PySequence_Concat");
   RESOLVE(gdbpy_PyErr_NewException, PyObject * (*) (const char *name, PyObject *base, PyObject *dict), "PyErr_NewException");
   RESOLVE(gdbpy_PyString_FromStringAndSize, PyObject * (*) (const char *, Py_ssize_t), "PyString_FromStringAndSize");
   RESOLVE(gdbpy_PyBytes_FromStringAndSize, PyObject * (*) (const char *, Py_ssize_t), "PyBytes_FromStringAndSize");
 #ifdef IS_PY3K
+  RESOLVE(gdb_PyBytes_AsString, char * (*) (PyObject *), "PyBytes_AsString");
+  RESOLVE(gdbpy_PyBytes_FromString, PyObject * (*) (const char *), "PyBytes_FromString");
+  RESOLVE(gdb_PyBytes_FromString, PyObject * (*) (const char *), "PyBytes_FromString");
   RESOLVE(gdbpy_PyMemoryView_FromObject, PyObject * (*) (PyObject *), "PyMemoryView_FromObject");
   RESOLVE(gdbpy_PySlice_GetIndicesEx, int (*) (PyObject *slice, Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step, Py_ssize_t *slicelength), "PySlice_GetIndiciesEx");
+  RESOLVE(gdbpy_SetProgramName, void (*) (wchar_t *), "Py_SetProgramName");
 #else
   RESOLVE(gdbpy_PyBuffer_FromObject, PyObject * (*) (PyObject *, Py_ssize_t offset, Py_ssize_t size), "PyBuffer_FromObject");
   RESOLVE(gdbpy_PySlice_GetIndicesEx, int (*) (PySliceObject *slice, Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step, Py_ssize_t *slicelength), "PySlice_GetIndiciesEx");
+  RESOLVE(gdbpy_SetProgramName, void (*) (char *), "Py_SetProgramName");
 #endif
   RESOLVE(gdbpy_PySlice_Check, int (*) (PyObject *slice), "PySlice_Check");
 
@@ -390,7 +541,7 @@ extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
   RESOLVE(gdb_PyErr_Clear, void (*) (void), "PyErr_Clear");
   RESOLVE(gdb_PyErr_ExceptionMatches, int (*) (PyObject *), "PyErr_ExceptionMatches");
   RESOLVE(gdb_PyErr_Fetch, void (*) (PyObject **, PyObject **, PyObject **), "PyErr_Fetch");
-  RESOLVE(gdb_PyErr_GivenExceptionMatches, int (*) (PyObject *, PyObject *), "PyErr_GivenExceptionMatch");
+  RESOLVE(gdb_PyErr_GivenExceptionMatches, int (*) (PyObject *, PyObject *), "PyErr_GivenExceptionMatches");
   RESOLVE(gdb_PyErr_Occurred, PyObject * (*) (void), "PyErr_Occurred");
   RESOLVE(gdb_PyErr_Print, void (*) (void), "PyErr_Print");
   RESOLVE(gdb_PyErr_Restore, void (*) (PyObject *, PyObject *, PyObject *), "PyErr_Restore");
@@ -409,7 +560,12 @@ extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
   RESOLVE(gdb_PyGILState_Release, void (*) (PyGILState_STATE), "PyGILState_Release");
   RESOLVE(gdb_PyImport_AddModule, PyObject * (*) (const char *), "PyImport_AddModule");
   RESOLVE(gdb_PyImport_ImportModule, PyObject * (*) (const char *), "PyImport_ImportModule");
+#ifdef IS_PY3K
+  RESOLVE(gdb_PyImport_AppendInittab, int (*) (const char *, PyObject *(*)(void)), "PyImport_AppendInittab");
+  RESOLVE(gdb_PyBuffer_FillInfo, int (*) (Py_buffer *view, PyObject *exporter, void *buf, Py_ssize_t len, int readonly, int flags), "PyBuffer_FillInfo");
+#else
   RESOLVE(gdb_PyInt_AsLong, long (*) (PyObject *), "PyInt_AsLong");
+#endif
   RESOLVE(gdb_PyInt_FromLong, PyObject * (*) (long), "PyInt_FromLong");
   RESOLVE(gdb_PyInt_GetMax, long (*) (void), "PyInt_GetMax");
   RESOLVE(gdb_PyIter_Next, PyObject * (*) (PyObject *), "PyIter_Next");
@@ -420,12 +576,18 @@ extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
   RESOLVE(gdb_PyList_New, PyObject * (*) (Py_ssize_t size), "PyList_New");
   RESOLVE(gdb_PyList_Size, Py_ssize_t (*) (PyObject *), "PyList_Size");
   RESOLVE(gdb_PyLong_AsLongLong, PY_LONG_LONG (*) (PyObject *), "PyLong_AsLongLong");
+#ifdef IS_PY3K
+  RESOLVE(gdb_PyLong_AsLong, long (*) (PyObject *), "PyLong_AsLong");
+#endif
   RESOLVE(gdb_PyLong_AsUnsignedLongLong, unsigned PY_LONG_LONG (*) (PyObject *), "PyLong_AsUnsignedLongLong");
   RESOLVE(gdb_PyLong_FromLong, PyObject * (*) (long), "PyLong_FromLong");
   RESOLVE(gdb_PyLong_FromLongLong, PyObject * (*) (PY_LONG_LONG), "PyLong_FromLongLong");
   RESOLVE(gdb_PyLong_FromUnsignedLong, PyObject * (*) (unsigned long), "PyLong_FromUnsignedLong");
   RESOLVE(gdb_PyLong_FromUnsignedLongLong, PyObject * (*) (unsigned PY_LONG_LONG), "PyLong_FromUnsignedLongLong");
   RESOLVE(gdb_PyMem_Malloc, void * (*) (size_t), "PyMem_Malloc");
+#ifdef IS_PY3K
+  RESOLVE(gdb_PyMem_RawMalloc, void * (*) (size_t), "PyMem_RawMalloc");
+#endif
   RESOLVE(gdb_PyModule_AddIntConstant, int (*) (PyObject *, const char *, long), "PyModule_AddIntConstant");
   RESOLVE(gdb_PyModule_AddObject, int (*) (PyObject *, const char *, PyObject *), "PyModule_AddObject");
   RESOLVE(gdb_PyModule_AddStringConstant, int (*) (PyObject *, const char *, const char *), "PyModule_AddStringConstant");
@@ -475,14 +637,28 @@ extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
   RESOLVE(gdb_Py_Initialize, void (*) (void), "Py_Initialize");
   RESOLVE(gdb_Py_InitModule4, PyObject * (*) (const char *, PyMethodDef *, const char *, PyObject *, int), "Py_InitModule4");
   RESOLVE(gdb_Py_InitModule4_64, PyObject * (*) (const char *, PyMethodDef *, const char *, PyObject *, int), "Py_InitModule4_64");
+#ifdef IS_PY3K
+  RESOLVE(gdb_PyModule_Create2, PyObject * (*)(PyModuleDef *, int), "PyModule_Create2");
+#endif
   RESOLVE(gdb_PyObject_Call, PyObject * (*) (PyObject *callable_object, PyObject *args, PyObject *kw), "PyObject_Call");
   RESOLVE(gdb_PyObject_CallObject, PyObject * (*) (PyObject *callable_object, PyObject *args), "PyObject_CallObject");
+#ifdef IS_PY3K
+  RESOLVE(gdb_Py_SetProgramName, void (*) (wchar_t *), "Py_SetProgramName");
+#else
   RESOLVE(gdb_Py_SetProgramName, void (*) (char *), "Py_SetProgramName");
+#endif
   RESOLVE(gdb__PyObject_New, PyObject * (*) (PyTypeObject *), "_PyObject_New");
   RESOLVE(gdb_PyCode_New, PyCodeObject * (*) (int, int, int, int,
 					      PyObject *, PyObject *, PyObject *, PyObject *,
 					      PyObject *, PyObject *, PyObject *, PyObject *, int, PyObject *), "PyCode_New");
   RESOLVE(gdb_PyFrame_New, PyFrameObject * (*) (PyThreadState *, PyCodeObject *, PyObject *, PyObject *), "PyFrame_New");
+#ifdef IS_PY3K
+  RESOLVE(gdb_PyUnicode_FromString, PyObject * (*)(const char *string), "PyUnicode_FromString");
+  RESOLVE(gdb_PyUnicode_CompareWithASCIIString, int (*)(PyObject *uni, const char *string), "PyUnicode_CompareWithASCIIString");
+  RESOLVE(gdb_PyUnicode_Decode, PyObject * (*) (const char *s, Py_ssize_t size, const char *encoding, const char *errors), "PyUnicode_Decode");
+  RESOLVE(gdb_PyUnicode_AsEncodedString, PyObject * (*) (register PyObject *unicode, const char *encoding, const char *errors), "PyUnicode_AsEncodedString");
+  RESOLVE(gdb_PyUnicode_FromEncodedObject, PyObject * (*) (register PyObject *obj, const char *encoding, const char *errors), "PyUnicode_FromEncodedObject");
+#else
 #ifdef __APPLE__
   RESOLVE(gdb_PyUnicode_Decode, PyObject * (*) (const char *s, Py_ssize_t size, const char *encoding, const char *errors), "PyUnicodeUCS2_Decode");
   RESOLVE(gdb_PyUnicode_AsEncodedString, PyObject * (*) (register PyObject *unicode, const char *encoding, const char *errors), "PyUnicodeUCS2_AsEncodedString");
@@ -491,6 +667,7 @@ extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
   RESOLVE(gdb_PyUnicode_Decode, PyObject * (*) (const char *s, Py_ssize_t size, const char *encoding, const char *errors), "PyUnicodeUCS4_Decode");
   RESOLVE(gdb_PyUnicode_AsEncodedString, PyObject * (*) (register PyObject *unicode, const char *encoding, const char *errors), "PyUnicodeUCS4_AsEncodedString");
   RESOLVE(gdb_PyUnicode_FromEncodedObject, PyObject * (*) (register PyObject *obj, const char *encoding, const char *errors), "PyUnicodeUCS4_FromEncodedObject");
+#endif
 #endif
   RESOLVE(gdb_Py_DontWriteBytecodeFlag, int *, "Py_DontWriteBytecodeFlag");
   return true;
@@ -600,6 +777,35 @@ name (atype arg1, btype arg2, ctype arg3)                                     \
     warning ("%s: called while Python is not available!", __FUNCTION__);      \
 }
 
+#define PYWRAPPER_ARG5(rtype,name, atype, btype, ctype, dtype, etype)         \
+rtype                                                                         \
+name (atype arg1, btype arg2, ctype arg3, dtype arg4, etype arg5)             \
+{                                                                             \
+  if (!is_python_available () || gdb_## name == NULL)                         \
+    {                                                                         \
+      warning ("%s: called while Python is not available!", __FUNCTION__);    \
+      return (rtype) 0;                                                       \
+    }                                                                         \
+  return gdb_## name (arg1, arg2, arg3, arg4, arg5);                          \
+}
+#define PYWRAPPER_ARG6(rtype,name, atype, btype, ctype, dtype, etype, ftype)  \
+rtype                                                                         \
+name (atype arg1, btype arg2, ctype arg3, dtype arg4, etype arg5, ftype arg6) \
+{                                                                             \
+  if (!is_python_available () || gdb_## name == NULL)                         \
+    {                                                                         \
+      warning ("%s: called while Python is not available!", __FUNCTION__);    \
+      return (rtype) 0;                                                       \
+    }                                                                         \
+  return gdb_## name (arg1, arg2, arg3, arg4, arg5, arg6);                    \
+}
+
+#ifdef IS_PY3K
+PYWRAPPER_ARG6(int, PyBuffer_FillInfo, Py_buffer *, PyObject *, void *, Py_ssize_t, int, int)
+PYWRAPPER_ARG1(char *, PyBytes_AsString, PyObject *)
+PYWRAPPER_ARG1(PyObject *, PyBytes_FromString, const char *)
+#endif
+PYWRAPPERVOID_ARG1(PyBuffer_Release, Py_buffer *)
 PYWRAPPER_ARG1(PyObject *, PyBool_FromLong, long)
 PYWRAPPER_ARG1(int, PyCallable_Check, PyObject *)
 PYWRAPPER (PyObject *, PyDict_New)
@@ -625,8 +831,10 @@ PYWRAPPER (PyGILState_STATE, PyGILState_Ensure)
 PYWRAPPERVOID_ARG1(PyGILState_Release, PyGILState_STATE)
 PYWRAPPER_ARG1(PyObject *, PyImport_AddModule, CONSTCHAR *)
 PYWRAPPER_ARG1(PyObject *,PyImport_ImportModule, CONSTCHAR *)
+#ifndef IS_PY3K
 PYWRAPPER_ARG1(long, PyInt_AsLong, PyObject *)
 PYWRAPPER_ARG1(PyObject *,PyInt_FromLong, long)
+#endif
 PYWRAPPER (long, PyInt_GetMax)
 PYWRAPPER_ARG1(PyObject *, PyIter_Next, PyObject *)
 PYWRAPPER_ARG2(int, PyList_Append, PyObject *, PyObject *)
@@ -636,15 +844,25 @@ PYWRAPPER_ARG3(int, PyList_Insert, PyObject *, Py_ssize_t, PyObject *)
 PYWRAPPER_ARG1(PyObject *, PyList_New, Py_ssize_t)
 PYWRAPPER_ARG1(Py_ssize_t, PyList_Size, PyObject *)
 PYWRAPPER_ARG1(PY_LONG_LONG, PyLong_AsLongLong, PyObject *)
+#ifdef IS_PY3K
+PYWRAPPER_ARG1(long, PyLong_AsLong, PyObject *)
+#endif
 PYWRAPPER_ARG1(unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong, PyObject *)
 PYWRAPPER_ARG1(PyObject *, PyLong_FromLong, long)
 PYWRAPPER_ARG1(PyObject *, PyLong_FromLongLong, PY_LONG_LONG)
 PYWRAPPER_ARG1(PyObject *, PyLong_FromUnsignedLong, unsigned long)
 PYWRAPPER_ARG1(PyObject *, PyLong_FromUnsignedLongLong, unsigned PY_LONG_LONG)
+#ifdef IS_PY3K
+PYWRAPPER_ARG1(PyObject *, PyMemoryView_FromObject, PyObject *)
+PYWRAPPER_ARG1(void *, PyMem_RawMalloc, size_t)
+#endif
 PYWRAPPER_ARG1(void *, PyMem_Malloc, size_t)
 PYWRAPPER_ARG3(int, PyModule_AddIntConstant, PyObject *, CONSTCHAR *, long)
 PYWRAPPER_ARG3(int, PyModule_AddObject, PyObject *, CONSTCHAR *, PyObject *)
 PYWRAPPER_ARG3(int, PyModule_AddStringConstant, PyObject *, CONSTCHAR *, CONSTCHAR *)
+#ifdef IS_PY3K
+PYWRAPPER_ARG2(PyObject *, PyModule_Create2, PyModuleDef *, int)
+#endif
 PYWRAPPER_ARG1(PyObject *, PyModule_GetDict, PyObject *)
 PYWRAPPER_ARG1(PyObject *, PyNumber_Long, PyObject *)
 PYWRAPPER (int, PyOS_InterruptOccurred)
@@ -669,7 +887,9 @@ PYWRAPPER_ARG2(Py_ssize_t, PySequence_Index, PyObject *, PyObject *)
 PYWRAPPER_ARG1(PyObject *, PySequence_List, PyObject *)
 PYWRAPPER_ARG1(Py_ssize_t, PySequence_Size, PyObject *)
 PYWRAPPER_ARG1(char *, PyString_AsString, PyObject *)
+#ifndef IS_PY3K
 PYWRAPPER_ARG1(PyObject *, PyString_FromString, const char *)
+#endif
 PYWRAPPER_ARG1(Py_ssize_t, PyString_Size, PyObject *)
 //PYWRAPPER_ARG1(PyObject *, PySys_GetObject, char *)
 //PYWRAPPERVOID_ARG1(PySys_SetPath,char *)
@@ -683,16 +903,26 @@ PYWRAPPER_ARG2(int, PyType_IsSubtype, PyTypeObject *, PyTypeObject *)
 PYWRAPPER_ARG1(int, PyType_Ready, PyTypeObject *)
 PYWRAPPERVOID(Py_Finalize)
 PYWRAPPER(int, Py_FlushLine)
+#ifdef IS_PY3K
+PYWRAPPERVOID_ARG1(Py_SetProgramName, wchar_t *)
+#else
 PYWRAPPERVOID_ARG1(Py_SetProgramName, char *)
+#endif
 PYWRAPPER_ARG1(PyObject *, _PyObject_New, PyTypeObject *)
 PYWRAPPER_ARG2(PyObject *, PyObject_CallObject, PyObject *, PyObject *)
 PYWRAPPER_ARG3(PyObject *, PyObject_Call, PyObject *, PyObject *, PyObject *)
 //PYWRAPPER_ARG3(PyObject *, PyErr_NewException, char *, PyObject *, PyObject *)
+#ifndef IS_PY3K
 PYWRAPPER_ARG4(PyObject *, PyString_Decode, const char *, Py_ssize_t, const char *, const char *)
+#endif
 PYWRAPPER_ARG3(PyObject *, PyType_GenericNew, PyTypeObject *, PyObject *, PyObject *)
 PYWRAPPER_ARG3(int, PyObject_AsReadBuffer, PyObject *, const void **, Py_ssize_t *)
 PYWRAPPER_ARG3(int, PyObject_GenericSetAttr, PyObject *, PyObject *, PyObject *)
 PYWRAPPER_ARG3(PyObject *, PyBuffer_FromReadWriteObject, PyObject *, Py_ssize_t, Py_ssize_t)
+#ifdef IS_PY3K
+PYWRAPPER_ARG1(PyObject *, PyUnicode_FromString, const char *)
+PYWRAPPER_ARG2(int, PyUnicode_CompareWithASCIIString, PyObject *, const char *)
+#endif
 PYWRAPPER_ARG4 (PyObject *, PyUnicode_Decode, const char *, Py_ssize_t, const char *, const char *)
 PYWRAPPER_ARG3 (PyObject *, PyUnicode_FromEncodedObject, register PyObject *, const char *, const char *)
 PYWRAPPER_ARG3 (PyObject *,PyUnicode_AsEncodedString, register PyObject *, const char *, const char *)
@@ -752,6 +982,18 @@ Py_InitModule4(CONSTCHAR *name, PyMethodDef *methods, CONSTCHAR *doc, PyObject *
   return gdb_Py_InitModule4 (name, methods, doc, self, apiver);
 }
 
+#ifdef IS_PY3K
+int
+PyImport_AppendInittab(const char *name, PyObject *(*initfunc)(void))
+{
+  if (!is_python_available () || (gdb_PyImport_AppendInittab == NULL))
+    {
+      warning ("%s: called while Python is not available!", __FUNCTION__);
+      return -1;
+    }
+  return gdb_PyImport_AppendInittab(name, initfunc);
+}
+#endif
 
 #if HAVE_LIBPYTHON2_4
 PyObject *
