@@ -61,6 +61,37 @@
 #define QUALIFIERS static __forceinline__ __device__
 #endif
 
+
+#ifdef __CUDACC_RTC__
+#define CURAND_DETAIL_USE_CUDA_STL
+#endif
+
+#if __cplusplus >= 201103L
+#   ifdef CURAND_DETAIL_USE_CUDA_STL
+#       define CURAND_STD cuda::std
+#       include <cuda/std/type_traits>
+#   else
+#       define CURAND_STD std
+#       include <type_traits>
+#   endif // CURAND_DETAIL_USE_CUDA_STL
+#else
+// To support C++03 compilation
+#   define CURAND_STD curand_detail
+namespace curand_detail {
+    template<bool B, class T = void>
+    struct enable_if {};
+
+    template<class T>
+    struct enable_if<true, T> { typedef T type; };
+
+    template<class T, class U>
+    struct is_same { static const bool value = false; };
+
+    template<class T>
+    struct is_same<T, T> { static const bool value = true; };
+} // namespace curand_detail
+#endif // __cplusplus >= 201103L
+
 #ifndef __CUDACC_RTC__
 #include <math.h>
 #endif // __CUDACC_RTC__
@@ -276,6 +307,9 @@ QUALIFIERS void __curand_matvec_inplace(unsigned int *vector, unsigned int *matr
 {
     unsigned int result[N] = { 0 };
     for(int i = 0; i < N; i++) {
+        #ifdef __CUDA_ARCH__
+        #pragma unroll 16
+        #endif
         for(int j = 0; j < 32; j++) {
             if(vector[i] & (1 << j)) {
                 for(int k = 0; k < N; k++) {
@@ -1004,6 +1038,14 @@ QUALIFIERS unsigned long long __curand_umad(unsigned int a, unsigned int b, unsi
         : "=l"(r) : "r"(a), "r"(b), "l"(c));
     return r;
 }
+QUALIFIERS unsigned long long __curand_umul(unsigned int a, unsigned int b)
+{
+    unsigned long long r;
+    asm("mul.wide.u32 %0, %1, %2;"
+        : "=l"(r) : "r"(a), "r"(b));
+    return r;
+}
+
 QUALIFIERS double curand_MRG32k3a (curandStateMRG32k3a_t *state)
 {
     const unsigned int m1  = 4294967087u;
@@ -1016,22 +1058,29 @@ QUALIFIERS double curand_MRG32k3a (curandStateMRG32k3a_t *state)
     const unsigned int a23n = 1370589u;
 
     unsigned long long p1, p2;
-    p1 = __curand_umad(a12, state->s1[1], a13n * (unsigned long long)(m1 - state->s1[0]));
+    const unsigned long long p3 = __curand_umul(a13n, m1 - state->s1[0]);
+    p1 = __curand_umad(a12, state->s1[1], p3);
 
-    p1 = (p1 & 0xffffffff) + (p1 >> 32) * m1c;
+    // Putting addition inside and changing umul to umad
+    // slowed this function down on GV100
+    p1 =  __curand_umul(p1 >> 32, m1c) + (p1 & 0xffffffff);
     if (p1 >= m1) p1 -= m1;
 
     state->s1[0] = state->s1[1]; state->s1[1] = state->s1[2]; state->s1[2] = p1;
-    p2 = __curand_umad(a21, state->s2[2], a23n * (unsigned long long)(m2 - state->s2[0]));
+    const unsigned long long p4 = __curand_umul(a23n, m2 - state->s2[0]);
+    p2 = __curand_umad(a21, state->s2[2], p4);
 
-    p2 = (p2 & 0xffffffff) + (p2 >> 32) * m2c;
-    p2 = (p2 & 0xffffffff) + (p2 >> 32) * m2c;
+    // Putting addition inside and changing umul to umad
+    // slowed this function down on GV100
+    p2 =  __curand_umul(p2 >> 32, m2c) + (p2 & 0xffffffff);
+    p2 =  __curand_umul(p2 >> 32, m2c) + (p2 & 0xffffffff);
     if (p2 >= m2) p2 -= m2;
 
     state->s2[0] = state->s2[1]; state->s2[1] = state->s2[2]; state->s2[2] = p2;
 
-    if (p1 <= p2) return p1 - p2 + m1;
-    else return p1 - p2;
+    const unsigned int p5 = (unsigned int)p1 - (unsigned int)p2;
+    if(p1 <= p2) return p5 + m1;
+    return p5;
 }
 #elif __CUDA_ARCH__ > 0
 /*  nj's implementation */
@@ -1249,7 +1298,9 @@ QUALIFIERS void curand_init(unsigned long long seed,
  * \param state - Pointer to state to update
  */
 template <typename T>
-QUALIFIERS void skipahead(unsigned int n, T state)
+QUALIFIERS
+typename CURAND_STD::enable_if<CURAND_STD::is_same<curandStateSobol32_t*, T>::value || CURAND_STD::is_same<curandStateScrambledSobol32_t*, T>::value>::type
+skipahead(unsigned int n, T state)
 {
     unsigned int i_gray;
     state->x = state->c;
@@ -1275,7 +1326,9 @@ QUALIFIERS void skipahead(unsigned int n, T state)
  * \param state - Pointer to state to update
  */
 template <typename T>
-QUALIFIERS void skipahead(unsigned long long n, T state)
+QUALIFIERS
+typename CURAND_STD::enable_if<CURAND_STD::is_same<curandStateSobol64_t*, T>::value || CURAND_STD::is_same<curandStateScrambledSobol64_t*, T>::value>::type
+skipahead(unsigned long long n, T state)
 {
     unsigned long long i_gray;
     state->x = state->c;
