@@ -1,4 +1,4 @@
- /* Copyright 2009-2019 NVIDIA Corporation.  All rights reserved.
+ /* Copyright 2009-2021 NVIDIA CORPORATION & AFFILIATES.  All rights reserved.
   * 
   * NOTICE TO LICENSEE: 
   * 
@@ -626,13 +626,50 @@ typedef struct
 
 typedef struct
 {
-    unsigned int nMarkerLabelPixelCount;     /**< total number of pixels in this connected pixel region */
-    unsigned int nContourPixelCount;         /**< total number of pixels in this connected pixel region contour */
-    unsigned int nCurContourPixelIndex;      /**< current index of pixel being added to geometry info list */
+    Npp32u nMarkerLabelPixelCount;           /**< total number of pixels in this connected pixel region */
+    Npp32u nContourPixelCount;               /**< total number of pixels in this connected pixel region contour */
+    Npp32u nContourPixelsFound;              /**< total number of pixels in this connected pixel region contour found during geometry search */
     NppiPoint oContourFirstPixelLocation;    /**< image geometric x and y location of the first pixel in the contour */
     NppiRect oMarkerLabelBoundingBox;        /**< bounding box of this connected pixel region */
 } NppiCompressedMarkerLabelsInfo;
 
+
+/**
+ * Provides details of contour pixel grid map location and association 
+ */
+
+typedef struct
+{
+    Npp32u nMarkerLabelID;                   /**< this connected pixel region contour ID */
+    Npp32u nContourPixelCount;               /**< total number of pixels in this connected pixel region contour */
+    Npp32u nContourStartingPixelOffset;      /**< base offset of starting pixel in the overall contour pixel list */
+    Npp32u nSegmentNum;                      /**< relative block segment number within this particular contour ID */
+} NppiContourBlockSegment;
+
+
+/**
+ * Provides contour (boundary) geometry info of uniquely labeled pixel regions returned 
+ * by nppiCompressedMarkerLabelsUFInfo function in host memory in counterclockwise order relative to contour interiors. 
+ */
+
+typedef struct
+{
+    NppiPoint oContourOrderedGeometryLocation;                 /**< image geometry X and Y location in requested output order */
+    NppiPoint oContourPrevPixelLocation;                       /**< image geometry X and Y location of previous contour pixel */
+    NppiPoint oContourCenterPixelLocation;                     /**< image geometry X and Y location of center contour pixel */
+    NppiPoint oContourNextPixelLocation;                       /**< image geometry X and Y location of next contour pixel */
+    Npp32s nOrderIndex;                                        /**< contour pixel counterclockwise order index in geometry list */
+    Npp32s nReverseOrderIndex;                                 /**< contour pixel clockwise order index in geometry list */
+    Npp32u nFirstIndex;                                        /**< index of first ordered contour pixel in this subgroup */
+    Npp32u nLastIndex;                                         /**< index of last ordered contour pixel in this subgroup */
+    Npp32u nNextContourPixelIndex;                             /**< index of next ordered contour pixel in NppiContourPixelGeometryInfo list */
+    Npp32u nPrevContourPixelIndex;                             /**< index of previous ordered contour pixel in NppiContourPixelGeometryInfo list */
+    Npp8u nPixelAlreadyUsed;                                   /**< this test pixel is has already been used */
+    Npp8u nAlreadyLinked;                                      /**< this test pixel is already linked to center pixel */
+    Npp8u nAlreadyOutput;                                      /**< this pixel has already been output in geometry list */
+    Npp8u nContourInteriorDirection;                           /**< direction of contour region interior */
+} NppiContourPixelGeometryInfo;
+
 /**
  * Provides contour (boundary) direction info of uniquely labeled pixel regions returned 
  * by CompressedLabelMarkersUF function. 
@@ -650,35 +687,35 @@ typedef struct
 #define NPP_CONTOUR_DIRECTION_ANY_NORTH  NPP_CONTOUR_DIRECTION_NORTH_EAST | NPP_CONTOUR_DIRECTION_NORTH | NPP_CONTOUR_DIRECTION_NORTH_WEST
 #define NPP_CONTOUR_DIRECTION_ANY_WEST   NPP_CONTOUR_DIRECTION_NORTH_WEST | NPP_CONTOUR_DIRECTION_WEST | NPP_CONTOUR_DIRECTION_SOUTH_WEST
 #define NPP_CONTOUR_DIRECTION_ANY_SOUTH  NPP_CONTOUR_DIRECTION_SOUTH_EAST | NPP_CONTOUR_DIRECTION_SOUTH | NPP_CONTOUR_DIRECTION_SOUTH_WEST
-#define NPP_CONTOUR_DIRECtION_ANY_EAST   NPP_CONTOUR_DIRECTION_NORTH_EAST | NPP_CONTOUR_DIRECTION_EAST | NPP_CONTOUR_DIRECTION_SOUTH_EAST
+#define NPP_CONTOUR_DIRECTION_ANY_EAST   NPP_CONTOUR_DIRECTION_NORTH_EAST | NPP_CONTOUR_DIRECTION_EAST | NPP_CONTOUR_DIRECTION_SOUTH_EAST
 
 typedef struct
 {
-    unsigned char  nContourDirection;           /**< provides current contour pixel input and output direction info */
-    unsigned char  nContourInteriorDirection;   /**< provides current contour pixel region interior direction info */
-    unsigned short nReserved;                   /**< reserved for future use */
-    unsigned int   nMarkerLabelID;              /**< MarkerLabelID of contour interior connected region */
+    Npp32u  nMarkerLabelID;                         /**< MarkerLabelID of contour interior connected region */
+    Npp8u nContourDirectionCenterPixel;             /**< provides current center contour pixel input and output direction info */
+    Npp8u nContourInteriorDirectionCenterPixel;     /**< provides current center contour pixel region interior direction info */
+    Npp8u nConnected;                               /**< direction to directly connected contour pixels, 0 if not connected */
+    Npp8u nGeometryInfoIsValid;
+    NppiContourPixelGeometryInfo oContourPixelGeometryInfo;
+    NppiPoint nEast1, nNorthEast1, nNorth1, nNorthWest1, nWest1, nSouthWest1, nSouth1, nSouthEast1;
+    Npp8u aContourInteriorDirection1[4];
+    Npp8u nTest1EastConnected;
+    Npp8u nTest1NorthEastConnected;
+    Npp8u nTest1NorthConnected;
+    Npp8u nTest1NorthWestConnected;
+    Npp8u nTest1WestConnected;
+    Npp8u nTest1SouthWestConnected;
+    Npp8u nTest1SouthConnected;
+    Npp8u nTest1SouthEastConnected;
 } NppiContourPixelDirectionInfo;
 
 typedef struct
 {
-    unsigned int nTotalImagePixelContourCount;    /**< total number of contour pixels in image */
-    unsigned int nLongestImageContourPixelCount;  /**< longest per contour pixel count in image */
+    Npp32u nTotalImagePixelContourCount;    /**< total number of contour pixels in image */
+    Npp32u nLongestImageContourPixelCount;  /**< longest per contour pixel count in image */
 } NppiContourTotalsInfo;
 
 
-/**
- * Provides contour (boundary) direction and geometry info of uniquely labeled pixel regions returned 
- * by nppiCompressedMarkerLabelsUFInfo function in host memory. 
- */
-
-typedef struct
-{
-    NppiPoint oContourPixelLocation;                           /**< image geometry X and Y location of current contour pixel */
-    NppiContourPixelDirectionInfo oContourPixelDirectionInfo;  /**< information about region interior and connected contour pixels directions */
-} NppiContourPixelGeometryInfo;
-
-
 /**
  * Provides control of the type of segment boundaries, if any, added 
  * to the image generated by the watershed segmentation function. 
