@@ -11,7 +11,7 @@ class TESTReportSQLStatement(nsysstats.Report):
 
     usage = f"""{{SCRIPT}}[:<sql_statement>] -- Run SQL Statement
 
-    <sql_statement> : an arbitrary SQLite statement
+    <sql_statement> : Arbitrary SQLite statement
 
     Output defined by <sql_statement>.
 
