@@ -13,7 +13,7 @@ class TESTReportSQLTable(nsysstats.Report):
 
     usage = f"""{{SCRIPT}}[:<table_name>] -- Return Table
 
-    <table_name> : The name of an SQLite table
+    <table_name> : Name of an SQLite table
 
     Output defined by <table_name>.
 
